@@ -25,18 +25,30 @@ def close_groonga():
     groonga_process = None
     print '###<<< database: close'
 
+current_db_path = None
 def reconnect(name):
   global groonga_process
+  global current_db_path
   close_groonga()
-  db_path = os.path.join(DB_DIRECTORY, name)
-  if os.path.exists(db_path):
-    groonga_process = Popen(["groonga", db_path], stdin=PIPE, stdout=PIPE)
+  current_db_path = os.path.join(DB_DIRECTORY, name)
+  if os.path.exists(current_db_path):
+    groonga_process = Popen(["groonga", current_db_path],
+                            stdin=PIPE,
+                            stdout=PIPE)
   else:
-    groonga_process = Popen(["groonga", "-n", db_path], stdin=PIPE, stdout=PIPE)
-  print '###>>> database: open <%s>' % db_path
+    groonga_process = Popen(["groonga", "-n", current_db_path],
+                            stdin=PIPE,
+                            stdout=PIPE)
+  print '###>>> database: open <%s>' % current_db_path
+
+def expand_command_line(command_line):
+  return command_line.replace('${DB_PATH}', current_db_path)
 
 fout = None
 
+def normalize_file_name(file_name):
+  return re.sub('^.*?/lib/', 'lib/', file_name)
+
 def normalize_output(output):
   status = output[0]
   if status:
@@ -44,6 +56,13 @@ def normalize_output(output):
     normalized_elapsed_time = 0.000355720520019531
     status[1] = normalized_start_time
     status[2] = normalized_elapsed_time
+    return_code = status[0]
+    if return_code != 0:
+      backtraces = status[4]
+      if backtraces:
+        for backtrace in backtraces:
+          file_name = backtrace[1]
+          backtrace[1] = normalize_file_name(file_name)
   return output
 
 def remove_continuous_marks(command):
@@ -75,33 +94,37 @@ def execmd(command, fout):
     return
   output_buffer = ""
   first_timeout = 1
-  rest_timeout = 0.1
   timeout = first_timeout
   while True:
     out = select([groonga_process.stdout], [], [], timeout)
     timeout = rest_timeout
     if len(out[0]):
       char = groonga_process.stdout.read(1)
-      if char is None:
+      if char is None or char == '':
         stdout.write(output_buffer)
         if fout:
           fout.write(output_buffer)
+        break
       else:
         output_buffer += char
         if char == '\n':
-          parsed_output = json.loads(output_buffer)
-          normalized_output = normalize_output(parsed_output)
-          if len(output_buffer) < 80:
-            formatted_output = json.dumps(normalized_output,
-                                          ensure_ascii=False)
+          if command == 'dump':
+            formatted_output = output_buffer
+            stdout.write(output_buffer)
           else:
-            formatted_output = json.dumps(normalized_output,
-                                          indent=2,
-                                          ensure_ascii=False)
-            formatted_output += "\n"
-            formatted_output = formatted_output.encode("utf-8")
-          stdout.write(formatted_output)
-          stdout.write("\n")
+            parsed_output = json.loads(output_buffer)
+            normalized_output = normalize_output(parsed_output)
+            if len(output_buffer) < 80:
+              formatted_output = json.dumps(normalized_output,
+                                            ensure_ascii=False)
+            else:
+              formatted_output = json.dumps(normalized_output,
+                                            indent=2,
+                                            ensure_ascii=False)
+              formatted_output += "\n"
+              formatted_output = formatted_output.encode("utf-8")
+            stdout.write(formatted_output)
+            stdout.write("\n")
           if fout:
             if is_console:
               prefix = "  # "
@@ -159,7 +182,7 @@ def readfile(fname, outflag):
           if fout:
             fout.write("  " + command_line + "\n")
           print command_line
-          os.system(command_line)
+          os.system(expand_command_line(command_line))
         elif cmd.startswith('.. .. '):
           command_line = cmd[6:]
           if fout:
@@ -188,8 +211,12 @@ def readfile(fname, outflag):
       print '###<<< include end'
 
 entry_point = "source/"
+rest_timeout = 0.1
 if len(argv) == 2:
   entry_point = argv[1]
+elif len(argv) == 3:
+  entry_point = argv[1]
+  rest_timeout = float(argv[2])
 if os.path.isfile(entry_point):
   readfile(entry_point, 0)
 else:
