@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2014-2015 Brazil
+  Copyright(C) 2014-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,21 +15,27 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GROONGA_H
-#define GROONGA_H
+
+#pragma once
 
 #include "groonga/portability.h"
 #include "groonga/groonga.h"
-#include "groonga/conf.h"
-#include "groonga/obj.h"
-#include "groonga/ii.h"
-#include "groonga/geo.h"
+
+#include "groonga/array.h"
+#include "groonga/config.h"
+#include "groonga/dat.h"
+#include "groonga/dump.h"
 #include "groonga/expr.h"
+#include "groonga/file_reader.h"
+#include "groonga/geo.h"
+#include "groonga/hash.h"
+#include "groonga/ii.h"
+#include "groonga/obj.h"
 #include "groonga/output.h"
-#include "groonga/util.h"
+#include "groonga/pat.h"
 #include "groonga/request_canceler.h"
 #include "groonga/thread.h"
+#include "groonga/type.h"
+#include "groonga/util.h"
 #include "groonga/windows.h"
 #include "groonga/windows_event_logger.h"
-
-#endif /* GROONGA_H */
