@@ -124,7 +124,8 @@ typedef enum {
   GRN_TOKEN_FILTER_ERROR = -73,
   GRN_COMMAND_ERROR = -74,
   GRN_PLUGIN_ERROR = -75,
-  GRN_SCORER_ERROR = -76
+  GRN_SCORER_ERROR = -76,
+  GRN_CANCEL = -77
 } grn_rc;
 
 GRN_API grn_rc grn_init(void);
@@ -290,7 +291,7 @@ GRN_API grn_encoding grn_encoding_parse(const char *name);
 
 /* obj */
 
-typedef unsigned short int grn_obj_flags;
+typedef uint16_t grn_obj_flags;
 
 #define GRN_OBJ_TABLE_TYPE_MASK        (0x07)
 #define GRN_OBJ_TABLE_HASH_KEY         (0x00)
@@ -335,6 +336,8 @@ typedef unsigned short int grn_obj_flags;
 #define GRN_OBJ_UNIT_USERDEF_SECTION   (0x07<<8)
 #define GRN_OBJ_UNIT_USERDEF_POSITION  (0x08<<8)
 
+#define GRN_OBJ_KEY_LARGE              (0x01<<12)
+
 #define GRN_OBJ_NO_SUBREC              (0x00<<13)
 #define GRN_OBJ_WITH_SUBREC            (0x01<<13)
 
@@ -733,7 +736,8 @@ typedef enum {
   GRN_OP_TABLE_GROUP,
   GRN_OP_JSON_PUT,
   GRN_OP_GET_MEMBER,
-  GRN_OP_REGEXP
+  GRN_OP_REGEXP,
+  GRN_OP_FUZZY
 } grn_operator;
 
 GRN_API const char *grn_operator_to_string(grn_operator op);
@@ -892,6 +896,7 @@ GRN_API int grn_obj_get_values(grn_ctx *ctx, grn_obj *obj, grn_id offset, void *
 
 GRN_API grn_rc grn_obj_set_value(grn_ctx *ctx, grn_obj *obj, grn_id id, grn_obj *value, int flags);
 GRN_API grn_rc grn_obj_remove(grn_ctx *ctx, grn_obj *obj);
+GRN_API grn_rc grn_obj_remove_dependent(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc grn_obj_rename(grn_ctx *ctx, grn_obj *obj,
                               const char *name, unsigned int name_size);
 GRN_API grn_rc grn_table_rename(grn_ctx *ctx, grn_obj *table,
@@ -932,6 +937,18 @@ GRN_API grn_obj *grn_obj_db(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API grn_id grn_obj_id(grn_ctx *ctx, grn_obj *obj);
 
+/* Flags for grn_fuzzy_search_optarg::flags. */
+#define GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION                  (0x01)
+
+typedef struct _grn_fuzzy_search_optarg grn_fuzzy_search_optarg;
+
+struct _grn_fuzzy_search_optarg {
+  unsigned int max_distance;
+  unsigned int max_expansion;
+  unsigned int prefix_match_size;
+  int flags;
+};
+
 typedef struct _grn_search_optarg grn_search_optarg;
 
 struct _grn_search_optarg {
@@ -945,6 +962,7 @@ struct _grn_search_optarg {
   grn_obj *scorer;
   grn_obj *scorer_args_expr;
   unsigned int scorer_args_expr_offset;
+  grn_fuzzy_search_optarg fuzzy;
 };
 
 GRN_API grn_rc grn_obj_search(grn_ctx *ctx, grn_obj *obj, grn_obj *query,
