@@ -15,31 +15,28 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "groonga_in.h"
+#include "grn.h"
 
 #include <stdio.h>
 #include <string.h>
-#include "ctx_impl.h"
+#include "grn_ctx_impl.h"
 
 #ifdef WIN32
-#  include <ws2tcpip.h>
+# include <ws2tcpip.h>
 #else
-#  ifdef HAVE_SYS_SOCKET_H
-#    include <sys/socket.h>
-#  endif /* HAVE_SYS_SOCKET_H */
-#  ifdef HAVE_NETINET_IN_H
-#    include <netinet/in.h>
-#  endif /* HAVE_NETINET_IN_H */
-#  ifdef HAVE_NETINET_TCP_H
-#   include <netinet/tcp.h>
-#  endif /* HAVE_NETINET_TCP_H */
-#  ifdef HAVE_SIGNAL_H
-#   include <signal.h>
-#  endif /* HAVE_SIGNAL_H */
+# ifdef HAVE_SYS_SOCKET_H
+#  include <sys/socket.h>
+# endif /* HAVE_SYS_SOCKET_H */
+# include <netinet/in.h>
+# include <netinet/tcp.h>
+# ifdef HAVE_SIGNAL_H
+#  include <signal.h>
+# endif /* HAVE_SIGNAL_H */
+# include <sys/uio.h>
 #endif /* WIN32 */
 
-#include "ctx.h"
-#include "com.h"
+#include "grn_ctx.h"
+#include "grn_com.h"
 
 #ifndef PF_INET
 #define PF_INET AF_INET
@@ -193,7 +190,7 @@ grn_msg_send(grn_ctx *ctx, grn_obj *msg, int flags)
       {
         ssize_t ret;
         ret = send(peer->fd, GRN_BULK_HEAD(msg), GRN_BULK_VSIZE(msg), MSG_NOSIGNAL);
-        if (ret == -1) { SERR("send"); }
+        if (ret == -1) { SOERR("send"); }
         if (ctx->rc != GRN_OPERATION_WOULD_BLOCK) {
           grn_com_queue_enque(ctx, m->old, (grn_com_queue_entry *)msg);
           return ctx->rc;
@@ -204,7 +201,7 @@ grn_msg_send(grn_ctx *ctx, grn_obj *msg, int flags)
       {
         if ((flags & GRN_CTX_MORE)) { flags |= GRN_CTX_QUIET; }
         if (ctx->stat == GRN_CTX_QUIT) { flags |= GRN_CTX_QUIT; }
-        header->qtype = (uint8_t) ctx->impl->output_type;
+        header->qtype = (uint8_t) ctx->impl->output.type;
         header->keylen = 0;
         header->level = 0;
         header->flags = flags;
@@ -251,7 +248,7 @@ grn_com_init(void)
   WSADATA wd;
   if (WSAStartup(MAKEWORD(2, 0), &wd) != 0) {
     grn_ctx *ctx = &grn_gctx;
-    SERR("WSAStartup");
+    SOERR("WSAStartup");
   }
 #else /* WIN32 */
 #ifndef USE_MSG_NOSIGNAL
@@ -280,6 +277,10 @@ grn_com_event_init(grn_ctx *ctx, grn_com_event *ev, int max_nevents, int data_si
     MUTEX_INIT(ev->mutex);
     COND_INIT(ev->cond);
     GRN_COM_QUEUE_INIT(&ev->recv_old);
+    ev->msg_handler = NULL;
+    memset(&(ev->curr_edge_id), 0, sizeof(grn_com_addr));
+    ev->acceptor = NULL;
+    ev->opaque = NULL;
 #ifndef USE_SELECT
 # ifdef USE_EPOLL
     if ((ev->events = GRN_MALLOC(sizeof(struct epoll_event) * max_nevents))) {
@@ -310,7 +311,7 @@ grn_com_event_init(grn_ctx *ctx, grn_com_event *ev, int max_nevents, int data_si
     ev->hash = NULL;
     ev->events = NULL;
 #else /* USE_SELECT */
-
+    goto exit;
 #endif /* USE_SELECT */
   }
 exit :
@@ -328,10 +329,10 @@ grn_com_event_fin(grn_ctx *ctx, grn_com_event *ev)
 #ifndef USE_SELECT
   if (ev->events) { GRN_FREE(ev->events); }
 # ifdef USE_EPOLL
-  GRN_CLOSE(ev->epfd);
+  grn_close(ev->epfd);
 # endif /* USE_EPOLL */
 # ifdef USE_KQUEUE
-  GRN_CLOSE(ev->kqfd);
+  grn_close(ev->kqfd);
 # endif /* USE_KQUEUE*/
 #endif /* USE_SELECT */
   return GRN_SUCCESS;
@@ -351,7 +352,7 @@ grn_com_event_add(grn_ctx *ctx, grn_com_event *ev, grn_sock fd, int events, grn_
     struct epoll_event e;
     memset(&e, 0, sizeof(struct epoll_event));
     e.data.fd = (fd);
-    e.events = (__uint32_t) events;
+    e.events = (uint32_t) events;
     if (epoll_ctl(ev->epfd, EPOLL_CTL_ADD, (fd), &e) == -1) {
       SERR("epoll_ctl");
       return ctx->rc;
@@ -399,7 +400,7 @@ grn_com_event_mod(grn_ctx *ctx, grn_com_event *ev, grn_sock fd, int events, grn_
       struct epoll_event e;
       memset(&e, 0, sizeof(struct epoll_event));
       e.data.fd = (fd);
-      e.events = (__uint32_t) events;
+      e.events = (uint32_t) events;
       if (epoll_ctl(ev->epfd, EPOLL_CTL_MOD, (fd), &e) == -1) {
         SERR("epoll_ctl");
         return ctx->rc;
@@ -474,7 +475,7 @@ grn_com_event_start_accept(grn_ctx *ctx, grn_com_event *ev)
     if (listen(com->fd, LISTEN_BACKLOG) == 0) {
       com->accepting = GRN_TRUE;
     } else {
-      SERR("listen - start accept");
+      SOERR("listen - start accept");
     }
   }
   GRN_API_RETURN(ctx->rc);
@@ -492,7 +493,7 @@ grn_com_event_stop_accept(grn_ctx *ctx, grn_com_event *ev)
     if (listen(com->fd, 0) == 0) {
       com->accepting = GRN_FALSE;
     } else {
-      SERR("listen - disable accept");
+      SOERR("listen - disable accept");
     }
   }
   GRN_API_RETURN(ctx->rc);
@@ -510,7 +511,7 @@ grn_com_receiver(grn_ctx *ctx, grn_com *com)
       if (errno == EMFILE) {
         grn_com_event_stop_accept(ctx, ev);
       } else {
-        SERR("accept");
+        SOERR("accept");
       }
       return;
     }
@@ -528,7 +529,7 @@ grn_com_receiver(grn_ctx *ctx, grn_com *com)
     grn_msg *msg = (grn_msg *)grn_msg_open(ctx, com, &ev->recv_old);
     grn_com_recv(ctx, msg->u.peer, &msg->header, (grn_obj *)msg);
     if (msg->u.peer /* is_edge_request(msg)*/) {
-      memcpy(&msg->edge_id, &ev->curr_edge_id, sizeof(grn_com_addr));
+      grn_memcpy(&msg->edge_id, &ev->curr_edge_id, sizeof(grn_com_addr));
       if (!com->has_sid) {
         com->has_sid = 1;
         com->sid = ev->curr_edge_id.sid++;
@@ -560,14 +561,29 @@ grn_com_event_poll(grn_ctx *ctx, grn_com_event *ev, int timeout)
   FD_ZERO(&wfds);
   ctx->errlvl = GRN_OK;
   ctx->rc = GRN_SUCCESS;
-  GRN_HASH_EACH(ctx, ev->hash, eh, &pfd, &dummy, &com, {
-    if ((com->events & GRN_COM_POLLIN)) { FD_SET(*pfd, &rfds); }
-    if ((com->events & GRN_COM_POLLOUT)) { FD_SET(*pfd, &wfds); }
-    if (*pfd > nfds) { nfds = *pfd; }
-  });
+  {
+    grn_hash_cursor *cursor;
+    cursor = grn_hash_cursor_open(ctx, ev->hash, NULL, 0, NULL, 0, 0, -1, 0);
+    if (cursor) {
+      grn_id id;
+      while ((id = grn_hash_cursor_next(ctx, cursor))) {
+        grn_hash_cursor_get_key_value(ctx,
+                                      cursor,
+                                      (void **)(&pfd),
+                                      &dummy,
+                                      (void **)(&com));
+        if ((com->events & GRN_COM_POLLIN)) { FD_SET(*pfd, &rfds); }
+        if ((com->events & GRN_COM_POLLOUT)) { FD_SET(*pfd, &wfds); }
+# ifndef WIN32
+        if (*pfd > nfds) { nfds = *pfd; }
+# endif /* WIN32 */
+      }
+      grn_hash_cursor_close(ctx, cursor);
+    }
+  }
   nevents = select(nfds + 1, &rfds, &wfds, NULL, (timeout >= 0) ? &tv : NULL);
   if (nevents < 0) {
-    SERR("select");
+    SOERR("select");
     if (ctx->rc == GRN_INTERRUPTED_FUNCTION_CALL) { ERRCLR(ctx); }
     return ctx->rc;
   }
@@ -636,7 +652,7 @@ grn_com_event_poll(grn_ctx *ctx, grn_com_event *ev, int timeout)
       e.data.fd = efd;
       e.events = ep->events;
       if (epoll_ctl(ev->epfd, EPOLL_CTL_DEL, efd, &e) == -1) { SERR("epoll_ctl"); }
-      if (grn_sock_close(efd) == -1) { SERR("close"); }
+      if (grn_sock_close(efd) == -1) { SOERR("close"); }
       continue;
     }
     if ((ep->events & GRN_COM_POLLIN)) { grn_com_receiver(ctx, com); }
@@ -650,7 +666,7 @@ grn_com_event_poll(grn_ctx *ctx, grn_com_event *ev, int timeout)
       GRN_LOG(ctx, GRN_LOG_ERROR, "fd(%d) not found in ev->set", efd);
       EV_SET(&e, efd, ep->filter, EV_DELETE, 0, 0, NULL);
       if (kevent(ev->kqfd, &e, 1, NULL, 0, NULL) == -1) { SERR("kevent"); }
-      if (grn_sock_close(efd) == -1) { SERR("close"); }
+      if (grn_sock_close(efd) == -1) { SOERR("close"); }
       continue;
     }
     if ((ep->filter == GRN_COM_POLLIN)) { grn_com_receiver(ctx, com); }
@@ -660,7 +676,7 @@ grn_com_event_poll(grn_ctx *ctx, grn_com_event *ev, int timeout)
     nevents--;
     if (!grn_hash_get(ctx, ev->hash, &efd, sizeof(grn_sock), (void *)&com)) {
       GRN_LOG(ctx, GRN_LOG_ERROR, "fd(%d) not found in ev->hash", efd);
-      if (grn_sock_close(efd) == -1) { SERR("close"); }
+      if (grn_sock_close(efd) == -1) { SOERR("close"); }
       continue;
     }
     if ((ep->revents & GRN_COM_POLLIN)) { grn_com_receiver(ctx, com); }
@@ -687,7 +703,7 @@ grn_com_send_http(grn_ctx *ctx, grn_com *cs, const char *path, uint32_t path_len
   GRN_TEXT_PUTS(ctx, &buf, " HTTP/1.0\r\n\r\n");
   // todo : refine
   if ((ret = send(cs->fd, GRN_BULK_HEAD(&buf), GRN_BULK_VSIZE(&buf), MSG_NOSIGNAL|flags)) == -1) {
-    SERR("send");
+    SOERR("send");
   }
   if (ret != GRN_BULK_VSIZE(&buf)) {
     GRN_LOG(ctx, GRN_LOG_NOTICE, "send %d != %d", (int)ret, (int)GRN_BULK_VSIZE(&buf));
@@ -715,31 +731,29 @@ grn_com_send(grn_ctx *ctx, grn_com *cs,
     wsabufs[1].buf = (char *)body;
     wsabufs[1].len = size;
     if (WSASend(cs->fd, wsabufs, 2, &n_sent, 0, NULL, NULL) == SOCKET_ERROR) {
-      SERR("WSASend");
+      SOERR("WSASend");
     }
     ret = n_sent;
 #else /* WIN32 */
     struct iovec msg_iov[2];
     struct msghdr msg;
+    memset(&msg, 0, sizeof(struct msghdr));
     msg.msg_name = NULL;
     msg.msg_namelen = 0;
     msg.msg_iov = msg_iov;
     msg.msg_iovlen = 2;
-    msg.msg_control = NULL;
-    msg.msg_controllen = 0;
-    msg.msg_flags = 0;
     msg_iov[0].iov_base = header;
     msg_iov[0].iov_len = sizeof(grn_com_header);
     msg_iov[1].iov_base = (char *)body;
     msg_iov[1].iov_len = size;
     if ((ret = sendmsg(cs->fd, &msg, MSG_NOSIGNAL|flags)) == -1) {
-      SERR("sendmsg");
+      SOERR("sendmsg");
       rc = ctx->rc;
     }
 #endif /* WIN32 */
   } else {
     if ((ret = send(cs->fd, (const void *)header, whole_size, MSG_NOSIGNAL|flags)) == -1) {
-      SERR("send");
+      SOERR("send");
       rc = ctx->rc;
     }
   }
@@ -787,7 +801,7 @@ grn_com_recv_text(grn_ctx *ctx, grn_com *com,
   for (;;) {
     if (grn_bulk_reserve(ctx, buf, BUFSIZE)) { return ctx->rc; }
     if ((ret = recv(com->fd, GRN_BULK_CURR(buf), BUFSIZE, 0)) < 0) {
-      SERR("recv text");
+      SOERR("recv text");
       if (ctx->rc == GRN_OPERATION_WOULD_BLOCK ||
           ctx->rc == GRN_INTERRUPTED_FUNCTION_CALL) {
         ERRCLR(ctx);
@@ -819,7 +833,7 @@ exit :
     GRN_BULK_REWIND(buf);
     grn_bulk_reserve(ctx, buf, BUFSIZE);
     if ((ret = recv(com->fd, GRN_BULK_CURR(buf), BUFSIZE, 0)) < 0) {
-      SERR("recv text body");
+      SOERR("recv text body");
     } else {
       GRN_BULK_CURR(buf) += ret;
     }
@@ -837,7 +851,7 @@ grn_com_recv(grn_ctx *ctx, grn_com *com, grn_com_header *header, grn_obj *buf)
   size_t rest = sizeof(grn_com_header);
   do {
     if ((ret = recv(com->fd, p, rest, 0)) < 0) {
-      SERR("recv size");
+      SOERR("recv size");
       GRN_LOG(ctx, GRN_LOG_ERROR, "recv error (%" GRN_FMT_SOCKET ")", com->fd);
       if (ctx->rc == GRN_OPERATION_WOULD_BLOCK ||
           ctx->rc == GRN_INTERRUPTED_FUNCTION_CALL) {
@@ -859,8 +873,8 @@ grn_com_recv(grn_ctx *ctx, grn_com *com, grn_com_header *header, grn_obj *buf)
     }
   } while (rest);
   GRN_LOG(ctx, GRN_LOG_INFO,
-          "recv (%lu,%x,%d,%02x,%02x,%04x)",
-          ntohl(header->size),
+          "recv (%u,%x,%d,%02x,%02x,%04x)",
+          (uint32_t)ntohl(header->size),
           header->flags,
           header->proto,
           header->qtype,
@@ -881,7 +895,7 @@ grn_com_recv(grn_ctx *ctx, grn_com *com, grn_com_header *header, grn_obj *buf)
       retry = 0;
       for (rest = value_size; rest;) {
         if ((ret = recv(com->fd, GRN_BULK_CURR(buf), rest, MSG_WAITALL)) < 0) {
-          SERR("recv body");
+          SOERR("recv body");
           if (ctx->rc == GRN_OPERATION_WOULD_BLOCK ||
               ctx->rc == GRN_INTERRUPTED_FUNCTION_CALL) {
             ERRCLR(ctx);
@@ -926,7 +940,8 @@ grn_com_copen(grn_ctx *ctx, grn_com_event *ev, const char *dest, int port)
 #ifdef AI_NUMERICSERV
   hints.ai_flags = AI_NUMERICSERV;
 #endif
-  snprintf(port_string, sizeof(port_string), "%d", port);
+  grn_snprintf(port_string, sizeof(port_string), sizeof(port_string),
+               "%d", port);
 
   getaddrinfo_result = getaddrinfo(dest, port_string, &hints, &addrinfo_list);
   if (getaddrinfo_result != 0) {
@@ -939,7 +954,7 @@ grn_com_copen(grn_ctx *ctx, grn_com_event *ev, const char *dest, int port)
 #endif
 #ifdef EAI_SYSTEM
     case EAI_SYSTEM:
-      SERR("getaddrinfo");
+      SOERR("getaddrinfo");
       break;
 #endif
     default:
@@ -956,13 +971,13 @@ grn_com_copen(grn_ctx *ctx, grn_com_event *ev, const char *dest, int port)
     fd = socket(addrinfo_ptr->ai_family, addrinfo_ptr->ai_socktype,
                 addrinfo_ptr->ai_protocol);
     if (fd == -1) {
-      SERR("socket");
+      SOERR("socket");
     } else if (setsockopt(fd, 6, TCP_NODELAY,
                           (const char *)&value, sizeof(value))) {
-      SERR("setsockopt");
+      SOERR("setsockopt");
       grn_sock_close(fd);
     } else if (connect(fd, addrinfo_ptr->ai_addr, addrinfo_ptr->ai_addrlen)) {
-      SERR("connect");
+      SOERR("connect");
       grn_sock_close(fd);
     } else {
       break;
@@ -995,9 +1010,9 @@ void
 grn_com_close_(grn_ctx *ctx, grn_com *com)
 {
   grn_sock fd = com->fd;
-  if (shutdown(fd, SHUT_RDWR) == -1) { /* SERR("shutdown"); */ }
+  if (shutdown(fd, SHUT_RDWR) == -1) { /* SOERR("shutdown"); */ }
   if (grn_sock_close(fd) == -1) {
-    SERR("close");
+    SOERR("close");
   } else {
     com->closed = 1;
   }
@@ -1034,7 +1049,8 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
   if (!bind_address) {
     bind_address = "0.0.0.0";
   }
-  snprintf(port_string, sizeof(port_string), "%d", port);
+  grn_snprintf(port_string, sizeof(port_string), sizeof(port_string),
+               "%d", port);
   memset(&hints, 0, sizeof(struct addrinfo));
   hints.ai_family = PF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
@@ -1054,7 +1070,7 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
 #endif
 #ifdef EAI_SYSTEM
     case EAI_SYSTEM:
-      SERR("getaddrinfo");
+      SOERR("getaddrinfo");
       break;
 #endif
     default:
@@ -1066,29 +1082,29 @@ grn_com_sopen(grn_ctx *ctx, grn_com_event *ev,
     goto exit;
   }
   if ((lfd = socket(bind_address_info->ai_family, SOCK_STREAM, 0)) == -1) {
-    SERR("socket");
+    SOERR("socket");
     goto exit;
   }
-  memcpy(&ev->curr_edge_id.addr, he->h_addr, he->h_length);
+  grn_memcpy(&ev->curr_edge_id.addr, he->h_addr, he->h_length);
   ev->curr_edge_id.port = htons(port);
   ev->curr_edge_id.sid = 0;
   {
     int v = 1;
     if (setsockopt(lfd, SOL_TCP, TCP_NODELAY, (void *) &v, sizeof(int)) == -1) {
-      SERR("setsockopt");
+      SOERR("setsockopt");
       goto exit;
     }
     if (setsockopt(lfd, SOL_SOCKET, SO_REUSEADDR, (void *) &v, sizeof(int)) == -1) {
-      SERR("setsockopt");
+      SOERR("setsockopt");
       goto exit;
     }
   }
   if (bind(lfd, bind_address_info->ai_addr, bind_address_info->ai_addrlen) < 0) {
-    SERR("bind");
+    SOERR("bind");
     goto exit;
   }
   if (listen(lfd, LISTEN_BACKLOG) < 0) {
-    SERR("listen");
+    SOERR("listen");
     goto exit;
   }
   if (ev) {
