@@ -305,7 +305,8 @@ grn_get_global_error_message(void)
 inline static void
 grn_alloc_info_set_backtrace(char *buffer, size_t size)
 {
-#  define N_TRACE_LEVEL 100
+#  ifdef HAVE_BACKTRACE
+#    define N_TRACE_LEVEL 100
   static void *trace[N_TRACE_LEVEL];
   char **symbols;
   int i, n, rest;
@@ -334,11 +335,15 @@ grn_alloc_info_set_backtrace(char *buffer, size_t size)
   } else {
     buffer[0] = '\0';
   }
-#  undef N_TRACE_LEVEL
+#    undef N_TRACE_LEVEL
+#  else /* HAVE_BACKTRACE */
+  buffer[0] = '\0';
+#  endif /* HAVE_BACKTRACE */
 }
 
 inline static void
-grn_alloc_info_add(void *address, const char *file, int line, const char *func)
+grn_alloc_info_add(void *address, size_t size,
+                   const char *file, int line, const char *func)
 {
   grn_ctx *ctx;
   grn_alloc_info *new_alloc_info;
@@ -348,6 +353,7 @@ grn_alloc_info_add(void *address, const char *file, int line, const char *func)
 
   new_alloc_info = malloc(sizeof(grn_alloc_info));
   new_alloc_info->address = address;
+  new_alloc_info->size = size;
   new_alloc_info->freed = GRN_FALSE;
   grn_alloc_info_set_backtrace(new_alloc_info->alloc_backtrace,
                                sizeof(new_alloc_info->alloc_backtrace));
@@ -367,7 +373,7 @@ grn_alloc_info_add(void *address, const char *file, int line, const char *func)
 }
 
 inline static void
-grn_alloc_info_change(void *old_address, void *new_address)
+grn_alloc_info_change(void *old_address, void *new_address, size_t size)
 {
   grn_ctx *ctx;
   grn_alloc_info *alloc_info;
@@ -379,6 +385,7 @@ grn_alloc_info_change(void *old_address, void *new_address)
   for (; alloc_info; alloc_info = alloc_info->next) {
     if (alloc_info->address == old_address) {
       alloc_info->address = new_address;
+      alloc_info->size = size;
       grn_alloc_info_set_backtrace(alloc_info->alloc_backtrace,
                                    sizeof(alloc_info->alloc_backtrace));
     }
@@ -397,11 +404,13 @@ grn_alloc_info_dump(grn_ctx *ctx)
   alloc_info = ctx->impl->alloc_info;
   for (; alloc_info; alloc_info = alloc_info->next) {
     if (alloc_info->freed) {
-      printf("address[%d][freed]: %p\n", i, alloc_info->address);
+      printf("address[%d][freed]: %p(%" GRN_FMT_SIZE ")\n",
+             i, alloc_info->address, alloc_info->size);
     } else {
-      printf("address[%d][not-freed]: %p: %s:%d: %s()\n%s",
+      printf("address[%d][not-freed]: %p(%" GRN_FMT_SIZE "): %s:%d: %s()\n%s",
              i,
              alloc_info->address,
+             alloc_info->size,
              alloc_info->file ? alloc_info->file : "(unknown)",
              alloc_info->line,
              alloc_info->func ? alloc_info->func : "(unknown)",
@@ -426,8 +435,12 @@ grn_alloc_info_check(void *address)
     if (alloc_info->address == address) {
       if (alloc_info->freed) {
         GRN_LOG(ctx, GRN_LOG_WARNING,
-                "double free: (%p):\nalloc backtrace:\n%sfree backtrace:\n%s",
+                "double free: %p(%" GRN_FMT_SIZE "):\n"
+                "alloc backtrace:\n"
+                "%sfree backtrace:\n"
+                "%s",
                 alloc_info->address,
+                alloc_info->size,
                 alloc_info->alloc_backtrace,
                 alloc_info->free_backtrace);
       } else {
@@ -461,13 +474,19 @@ grn_alloc_info_free(grn_ctx *ctx)
 }
 
 #else /* USE_MEMORY_DEBUG */
-#  define grn_alloc_info_add(address, file, line, func)
-#  define grn_alloc_info_change(old_address, new_address)
+#  define grn_alloc_info_add(address, size, file, line, func)
+#  define grn_alloc_info_change(old_address, new_address, size)
 #  define grn_alloc_info_check(address)
 #  define grn_alloc_info_dump(ctx)
 #  define grn_alloc_info_free(ctx)
 #endif /* USE_MEMORY_DEBUG */
 
+void
+grn_debug_dump_alloc_info(grn_ctx *ctx)
+{
+  grn_alloc_info_dump(ctx);
+}
+
 #ifdef USE_FAIL_MALLOC
 int grn_fmalloc_prob = 0;
 char *grn_fmalloc_func = NULL;
@@ -507,6 +526,7 @@ grn_loader_init(grn_loader *loader)
   GRN_TEXT_INIT(&loader->values, 0);
   GRN_UINT32_INIT(&loader->level, GRN_OBJ_VECTOR);
   GRN_PTR_INIT(&loader->columns, GRN_OBJ_VECTOR, GRN_ID_NIL);
+  loader->id_offset = -1;
   loader->key_offset = -1;
   loader->table = NULL;
   loader->last = NULL;
@@ -515,6 +535,9 @@ grn_loader_init(grn_loader *loader)
   loader->values_size = 0;
   loader->nrecords = 0;
   loader->stat = GRN_LOADER_BEGIN;
+  loader->columns_status = GRN_LOADER_COLUMNS_UNSET;
+  loader->rc = GRN_SUCCESS;
+  loader->errbuf[0] = '\0';
 }
 
 void
@@ -597,6 +620,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->stack_curr = 0;
   ctx->impl->curr_expr = NULL;
   ctx->impl->qe_next = NULL;
+  GRN_TEXT_INIT(&ctx->impl->current_request_id, 0);
   ctx->impl->parser = NULL;
 
   GRN_TEXT_INIT(&ctx->impl->output.names, GRN_OBJ_VECTOR);
@@ -696,7 +720,6 @@ grn_ctx_init_internal(grn_ctx *ctx, int flags)
   if (grn_ctx_per_db) {
     ctx->flags |= GRN_CTX_PER_DB;
   }
-  if (ERRP(ctx, GRN_ERROR)) { return ctx->rc; }
   ctx->stat = GRN_CTX_INITED;
   ctx->encoding = grn_gctx.encoding;
   ctx->seqno = 0;
@@ -715,7 +738,7 @@ grn_ctx_init_internal(grn_ctx *ctx, int flags)
   ctx->errfunc = "";
   ctx->trace[0] = NULL;
   ctx->errbuf[0] = '\0';
-  return ctx->rc;
+  return GRN_SUCCESS;
 }
 
 grn_rc
@@ -727,6 +750,15 @@ grn_ctx_init(grn_ctx *ctx, int flags)
   if (rc == GRN_SUCCESS) {
     grn_ctx_impl_init(ctx);
     rc = ctx->rc;
+    if (rc != GRN_SUCCESS) {
+      grn_ctx_fin(ctx);
+      if (flags & GRN_CTX_ALLOCATED) {
+        CRITICAL_SECTION_ENTER(grn_glock);
+        ctx->next->prev = ctx->prev;
+        ctx->prev->next = ctx->next;
+        CRITICAL_SECTION_LEAVE(grn_glock);
+      }
+    }
   }
 
   return rc;
@@ -739,7 +771,6 @@ grn_ctx_open(int flags)
   if (ctx) {
     grn_ctx_init(ctx, flags|GRN_CTX_ALLOCATED);
     if (ERRP(ctx, GRN_ERROR)) {
-      grn_ctx_fin(ctx);
       GRN_GFREE(ctx);
       ctx = NULL;
     }
@@ -769,6 +800,7 @@ grn_ctx_fin(grn_ctx *ctx)
     if (ctx->impl->parser) {
       grn_expr_parser_close(ctx);
     }
+    GRN_OBJ_FIN(ctx, &ctx->impl->current_request_id);
     if (ctx->impl->values) {
 #ifndef USE_MEMORY_DEBUG
       grn_db_obj *o;
@@ -1143,8 +1175,10 @@ grn_rc
 grn_ctx_close(grn_ctx *ctx)
 {
   grn_rc rc = grn_ctx_fin(ctx);
+  CRITICAL_SECTION_ENTER(grn_glock);
   ctx->next->prev = ctx->prev;
   ctx->prev->next = ctx->next;
+  CRITICAL_SECTION_LEAVE(grn_glock);
   GRN_GFREE(ctx);
   return rc;
 }
@@ -1469,17 +1503,15 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
         }
       }
       if (GRN_TEXT_LEN(&request_id) > 0) {
+        GRN_TEXT_SET(ctx, &ctx->impl->current_request_id,
+                     GRN_TEXT_VALUE(&request_id),
+                     GRN_TEXT_LEN(&request_id));
         grn_request_canceler_register(ctx,
                                       GRN_TEXT_VALUE(&request_id),
                                       GRN_TEXT_LEN(&request_id));
       }
       ctx->impl->curr_expr = expr;
       grn_expr_exec(ctx, expr, 0);
-      if (GRN_TEXT_LEN(&request_id) > 0) {
-        grn_request_canceler_unregister(ctx,
-                                        GRN_TEXT_VALUE(&request_id),
-                                        GRN_TEXT_LEN(&request_id));
-      }
     } else {
       ERR(GRN_INVALID_ARGUMENT, "invalid command name: %.*s",
           command_name_size, command_name);
@@ -1575,6 +1607,9 @@ grn_ctx_qe_exec(grn_ctx *ctx, const char *str, uint32_t str_len)
     }
   }
   if (GRN_TEXT_LEN(&request_id) > 0) {
+    GRN_TEXT_SET(ctx, &ctx->impl->current_request_id,
+                 GRN_TEXT_VALUE(&request_id),
+                 GRN_TEXT_LEN(&request_id));
     grn_request_canceler_register(ctx,
                                   GRN_TEXT_VALUE(&request_id),
                                   GRN_TEXT_LEN(&request_id));
@@ -1694,6 +1729,13 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
       if (ctx->impl->qe_next) {
         ERRCLR(ctx);
       } else {
+        if (GRN_TEXT_LEN(&ctx->impl->current_request_id) > 0) {
+          grn_obj *request_id = &ctx->impl->current_request_id;
+          grn_request_canceler_unregister(ctx,
+                                          GRN_TEXT_VALUE(request_id),
+                                          GRN_TEXT_LEN(request_id));
+          GRN_BULK_REWIND(&ctx->impl->current_request_id);
+        }
         GRN_QUERY_LOG(ctx, GRN_QUERY_LOG_RESULT_CODE,
                       "<", "rc=%d", ctx->rc);
       }
@@ -1716,12 +1758,26 @@ unsigned int
 grn_ctx_recv(grn_ctx *ctx, char **str, unsigned int *str_len, int *flags)
 {
   if (!ctx) { return GRN_INVALID_ARGUMENT; }
+
+  *flags = 0;
+
   if (ctx->stat == GRN_CTX_QUIT) {
-    *str = NULL;
-    *str_len = 0;
-    *flags = GRN_CTX_QUIT;
-    return 0;
+    grn_bool have_buffer = GRN_FALSE;
+
+    if (ctx->impl &&
+        !ctx->impl->com &&
+        GRN_TEXT_LEN(ctx->impl->output.buf) > 0) {
+      have_buffer = GRN_TRUE;
+    }
+
+    *flags |= GRN_CTX_QUIT;
+    if (!have_buffer) {
+      *str = NULL;
+      *str_len = 0;
+      return 0;
+    }
   }
+
   GRN_API_ENTER;
   if (ctx->impl) {
     if (ctx->impl->com) {
@@ -1735,9 +1791,11 @@ grn_ctx_recv(grn_ctx *ctx, char **str, unsigned int *str_len, int *flags)
         *str_len = GRN_BULK_VSIZE(ctx->impl->output.buf);
         if (header.flags & GRN_CTX_QUIT) {
           ctx->stat = GRN_CTX_QUIT;
-          *flags = GRN_CTX_QUIT;
+          *flags |= GRN_CTX_QUIT;
         } else {
-          *flags = (header.flags & GRN_CTX_TAIL) ? 0 : GRN_CTX_MORE;
+          if (!(header.flags & GRN_CTX_TAIL)) {
+            *flags |= GRN_CTX_MORE;
+          }
         }
         ctx->impl->output.type = header.qtype;
         ctx->rc = (int16_t)ntohs(header.status);
@@ -2490,14 +2548,14 @@ grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
     void *res = malloc(size);
     if (res) {
       GRN_ADD_ALLOC_COUNT(1);
-      grn_alloc_info_add(res, file, line, func);
+      grn_alloc_info_add(res, size, file, line, func);
     } else {
       if (!(res = malloc(size))) {
         MERR("malloc fail (%" GRN_FMT_SIZE ")=%p (%s:%d) <%d>",
              size, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
-        grn_alloc_info_add(res, file, line, func);
+        grn_alloc_info_add(res, size, file, line, func);
       }
     }
     return res;
@@ -2512,14 +2570,14 @@ grn_calloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
     void *res = calloc(size, 1);
     if (res) {
       GRN_ADD_ALLOC_COUNT(1);
-      grn_alloc_info_add(res, file, line, func);
+      grn_alloc_info_add(res, size, file, line, func);
     } else {
       if (!(res = calloc(size, 1))) {
         MERR("calloc fail (%" GRN_FMT_SIZE ")=%p (%s:%d) <%d>",
              size, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
-        grn_alloc_info_add(res, file, line, func);
+        grn_alloc_info_add(res, size, file, line, func);
       }
     }
     return res;
@@ -2555,10 +2613,10 @@ grn_realloc_default(grn_ctx *ctx, void *ptr, size_t size, const char* file, int
       }
     }
     if (ptr) {
-      grn_alloc_info_change(ptr, res);
+      grn_alloc_info_change(ptr, res, size);
     } else {
       GRN_ADD_ALLOC_COUNT(1);
-      grn_alloc_info_add(res, file, line, func);
+      grn_alloc_info_add(res, size, file, line, func);
     }
   } else {
     if (!ptr) { return NULL; }
@@ -2584,13 +2642,13 @@ grn_strdup_default(grn_ctx *ctx, const char *s, const char* file, int line, cons
     char *res = grn_strdup_raw(s);
     if (res) {
       GRN_ADD_ALLOC_COUNT(1);
-      grn_alloc_info_add(res, file, line, func);
+      grn_alloc_info_add(res, strlen(res) + 1, file, line, func);
     } else {
       if (!(res = grn_strdup_raw(s))) {
         MERR("strdup(%p)=%p (%s:%d) <%d>", s, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
-        grn_alloc_info_add(res, file, line, func);
+        grn_alloc_info_add(res, strlen(res) + 1, file, line, func);
       }
     }
     return res;
