@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_ARRAY_HPP_
-#define GRN_DAT_ARRAY_HPP_
+#pragma once
 
 #include "dat.hpp"
 
@@ -25,7 +25,7 @@ namespace dat {
 
 // This class is used to detect an out-of-range access in debug mode.
 template <typename T>
-class Array {
+class GRN_DAT_API Array {
  public:
   Array() : ptr_(NULL), size_(0) {}
   Array(void *ptr, UInt32 size) : ptr_(static_cast<T *>(ptr)), size_(size) {
@@ -96,5 +96,3 @@ class Array {
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_ARRAY_HPP_
