@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/*
+  Copyright(C) 2011-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_DAT_COMMON_HPP_
-#define GRN_DAT_COMMON_HPP_
+#pragma once
 
 #ifndef _MSC_VER
 # include <stddef.h>
@@ -42,6 +42,12 @@
 # endif  // WIN32
 #endif  // GRN_DAT_API
 
+#ifdef WIN32
+# define grn_memcpy(dest, src, n) ::memcpy_s((dest), (n), (src), (n))
+#else  // WIN32
+# define grn_memcpy(dest, src, n) std::memcpy((dest), (src), (n))
+#endif  // WIN32
+
 namespace grn {
 namespace dat {
 
@@ -57,21 +63,10 @@ typedef ::uint32_t UInt32;
 typedef ::uint64_t UInt64;
 #endif  // _MSC_VER
 
-#ifndef UINT8_MAX
-const UInt8 UINT8_MAX = static_cast<UInt8>(0xFFU);
-#endif  // UINT8_MAX
-
-#ifndef UINT16_MAX
-const UInt16 UINT16_MAX = static_cast<UInt16>(0xFFFFU);
-#endif  // UINT16_MAX
-
-#ifndef UINT32_MAX
-const UInt32 UINT32_MAX = static_cast<UInt32>(0xFFFFFFFFU);
-#endif  // UINT32_MAX
-
-#ifndef UINT64_MAX
-const UInt64 UINT64_MAX = static_cast<UInt64>(0xFFFFFFFFFFFFFFFFULL);
-#endif  // UINT64_MAX
+const UInt8  MAX_UINT8  = static_cast<UInt8>(0xFFU);
+const UInt16 MAX_UINT16 = static_cast<UInt16>(0xFFFFU);
+const UInt32 MAX_UINT32 = static_cast<UInt32>(0xFFFFFFFFU);
+const UInt64 MAX_UINT64 = static_cast<UInt64>(0xFFFFFFFFFFFFFFFFULL);
 
 // If a key is a prefix of another key, such a key is associated with a special
 // terminal node which has TERMINAL_LABEL.
@@ -186,13 +181,6 @@ class Exception : public std::exception {
         what_(ex.what_) {}
   virtual ~Exception() throw() {}
 
-  virtual Exception &operator=(const Exception &ex) throw() {
-    file_ = ex.file_;
-    line_ = ex.line_;
-    what_ = ex.what_;
-    return *this;
-  }
-
   virtual ErrorCode code() const throw() = 0;
   virtual const char *file() const throw() {
     return file_;
@@ -221,11 +209,6 @@ class Error : public Exception {
       : Exception(ex) {}
   virtual ~Error() throw() {}
 
-  virtual Error &operator=(const Error &ex) throw() {
-      *static_cast<Exception *>(this) = ex;
-      return *this;
-  }
-
   virtual ErrorCode code() const throw() {
     return T;
   }
@@ -262,5 +245,3 @@ typedef Error<STATUS_ERROR> StatusError;
 
 }  // namespace dat
 }  // namespace grn
-
-#endif  // GRN_DAT_COMMON_HPP_
