@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2011 Brazil
+/* Copyright(C) 2011-2015 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -28,14 +28,20 @@
 #  undef max
 # endif  // max
 #else  // WIN32
-#include <fcntl.h>
-#include <sys/mman.h>
-#include <unistd.h>
+# include <fcntl.h>
+# include <sys/mman.h>
+# include <unistd.h>
 #endif  // WIN32
 
 #include <algorithm>
 #include <limits>
 
+#ifdef WIN32
+# define GRN_IO_FILE_CREATE_MODE (GENERIC_READ | GENERIC_WRITE)
+#else /* WIN32 */
+# define GRN_IO_FILE_CREATE_MODE 0644
+#endif /* WIN32 */
+
 namespace grn {
 namespace dat {
 
@@ -117,10 +123,19 @@ void FileImpl::swap(FileImpl *rhs) {
   std::swap(addr_, rhs->addr_);
 }
 
+void FileImpl::flush() {
+  if (!addr_) {
+    return;
+  }
+
+  BOOL succeeded = ::FlushViewOfFile(addr_, size_);
+  GRN_DAT_THROW_IF(IO_ERROR, !succeeded);
+}
+
 void FileImpl::create_(const char *path, UInt64 size) {
   if ((path != NULL) && (path[0] != '\0')) {
-    file_ = ::CreateFileA(path, GENERIC_READ | GENERIC_WRITE,
-                          FILE_SHARE_READ | FILE_SHARE_WRITE,
+    file_ = ::CreateFileA(path, GRN_IO_FILE_CREATE_MODE,
+                          FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
                           NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
     GRN_DAT_THROW_IF(IO_ERROR, file_ == INVALID_HANDLE_VALUE);
 
@@ -162,8 +177,8 @@ void FileImpl::open_(const char *path) {
   GRN_DAT_THROW_IF(IO_ERROR,
       static_cast<UInt64>(st.st_size) > std::numeric_limits< ::size_t>::max());
 
-  file_ = ::CreateFileA(path, GENERIC_READ | GENERIC_WRITE,
-                        FILE_SHARE_READ | FILE_SHARE_WRITE,
+  file_ = ::CreateFileA(path, GRN_IO_FILE_CREATE_MODE,
+                        FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
                         NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   GRN_DAT_THROW_IF(IO_ERROR, file_ == NULL);
 
@@ -187,12 +202,21 @@ void FileImpl::swap(FileImpl *rhs) {
   std::swap(length_, rhs->length_);
 }
 
+void FileImpl::flush() {
+  if (!addr_) {
+    return;
+  }
+
+  int result = ::msync(addr_, length_, MS_SYNC);
+  GRN_DAT_THROW_IF(IO_ERROR, result != 0);
+}
+
 void FileImpl::create_(const char *path, UInt64 size) {
   GRN_DAT_THROW_IF(PARAM_ERROR,
       size > static_cast<UInt64>(std::numeric_limits< ::off_t>::max()));
 
   if ((path != NULL) && (path[0] != '\0')) {
-    fd_ = ::open(path, O_RDWR | O_CREAT | O_TRUNC, 0666);
+    fd_ = ::open(path, O_RDWR | O_CREAT | O_TRUNC, GRN_IO_FILE_CREATE_MODE);
     GRN_DAT_THROW_IF(IO_ERROR, fd_ == -1);
 
     const ::off_t file_size = static_cast< ::off_t>(size);
