@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2015 Brazil
+/*
+  Copyright(C) 2009-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,16 +17,41 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifndef GRN_H
-#define GRN_H
+#pragma once
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-#if defined(WIN32) && defined(__GNUC__)
-# define __MINGW_MSVC_COMPAT_WARNINGS
-#endif /* defined(WIN32) && defined(__GNUC__) */
+#ifdef WIN32
+# ifdef __GNUC__
+#  define __MINGW_MSVC_COMPAT_WARNINGS
+# endif /* __GNUC__ */
+
+# ifdef __GNUC__
+#  include <w32api.h>
+#  define GRN_MINIMUM_WINDOWS_VERSION WindowsVista
+# else /* __GNUC__ */
+#  define GRN_MINIMUM_WINDOWS_VERSION 0x0600 /* Vista */
+# endif /* __GNUC__ */
+
+# ifdef WINVER
+#  undef WINVER
+# endif /* WINVER */
+# define WINVER GRN_MINIMUM_WINDOWS_VERSION
+# ifdef _WIN32_WINNT
+#  undef _WIN32_WINNT
+# endif /* _WIN32_WINNT */
+# define _WIN32_WINNT GRN_MINIMUM_WINDOWS_VERSION
+# ifdef NTDDI_VERSION
+#  undef NTDDI_VERSION
+# endif /* NTDDI_VERSION */
+# define NTDDI_VERSION GRN_MINIMUM_WINDOWS_VERSION
+
+# ifdef WIN32_LEAN_AND_MEAN
+#  undef WIN32_LEAN_AND_MEAN
+# endif /* WIN32_LEAN_AND_MEAN */
+#endif /* WIN32 */
 
 #ifdef __cplusplus
 # define __STDC_LIMIT_MACROS
@@ -54,23 +80,17 @@
 
 #ifdef WIN32
 # define GRN_API __declspec(dllexport)
-#ifdef GROONGA_MAIN
-# define GRN_VAR __declspec(dllimport)
-#else
-# define GRN_VAR __declspec(dllexport) extern
-#endif /* GROONGA_MAIN */
+# ifdef GROONGA_MAIN
+#  define GRN_VAR __declspec(dllimport)
+# else
+#  define GRN_VAR __declspec(dllexport) extern
+# endif /* GROONGA_MAIN */
 #else
 # define GRN_API
 # define GRN_VAR extern
 #endif
 
 #ifdef WIN32
-
-# if defined(__GNUC__) && !defined(WINVER)
-#  include <w32api.h>
-#  define WINVER WindowsVista
-# endif /* defined(__GNUC__) && !defined(WINVER) */
-
 # include <basetsd.h>
 # include <process.h>
 # include <winsock2.h>
@@ -202,10 +222,11 @@ typedef void * grn_thread_func_result;
   (pthread_create(&(thread), NULL, (func), (arg)))
 # define THREAD_JOIN(thread) (pthread_join(thread, NULL))
 typedef pthread_mutex_t grn_mutex;
-# define MUTEX_INIT(m)   pthread_mutex_init(&m, NULL)
-# define MUTEX_LOCK(m)   pthread_mutex_lock(&m)
-# define MUTEX_UNLOCK(m) pthread_mutex_unlock(&m)
-# define MUTEX_FIN(m)
+# define MUTEX_INIT(m)       pthread_mutex_init(&m, NULL)
+# define MUTEX_LOCK(m)       pthread_mutex_lock(&m)
+# define MUTEX_LOCK_CHECK(m) (MUTEX_LOCK(m) == 0)
+# define MUTEX_UNLOCK(m)     pthread_mutex_unlock(&m)
+# define MUTEX_FIN(m)        pthread_mutex_destroy(&m)
 # ifdef HAVE_PTHREAD_MUTEXATTR_SETPSHARED
 #  define MUTEX_INIT_SHARED(m) do {\
   pthread_mutexattr_t mutexattr;\
@@ -238,6 +259,7 @@ typedef pthread_cond_t grn_cond;
 # else
 #  define COND_INIT_SHARED COND_INIT
 # endif /* HAVE_PTHREAD_CONDATTR_SETPSHARE */
+# define COND_FIN(c)    pthread_cond_destroy(&c)
 
 typedef pthread_key_t grn_thread_key;
 # define THREAD_KEY_CREATE(key, destr)  pthread_key_create(key, destr)
@@ -295,10 +317,11 @@ typedef unsigned int grn_thread_func_result;
 #  define THREAD_JOIN(thread) \
   (WaitForSingleObject((HANDLE)(thread), INFINITE) == WAIT_FAILED)
 typedef HANDLE grn_mutex;
-#  define MUTEX_INIT(m)   ((m) = CreateMutex(0, FALSE, NULL))
-#  define MUTEX_LOCK(m)   WaitForSingleObject((m), INFINITE)
-#  define MUTEX_UNLOCK(m) ReleaseMutex(m)
-#  define MUTEX_FIN(m)    CloseHandle(m)
+#  define MUTEX_INIT(m)       ((m) = CreateMutex(0, FALSE, NULL))
+#  define MUTEX_LOCK(m)       WaitForSingleObject((m), INFINITE)
+#  define MUTEX_LOCK_CHECK(m) (MUTEX_LOCK(m) == WAIT_OBJECT_0)
+#  define MUTEX_UNLOCK(m)     ReleaseMutex(m)
+#  define MUTEX_FIN(m)        CloseHandle(m)
 typedef CRITICAL_SECTION grn_critical_section;
 #  define CRITICAL_SECTION_INIT(cs)  InitializeCriticalSection(&(cs))
 #  define CRITICAL_SECTION_ENTER(cs) EnterCriticalSection(&(cs))
@@ -371,6 +394,12 @@ typedef struct
   } \
 } while (0)
 
+#  define COND_FIN(c) do { \
+  CloseHandle((c).waiters_done_); \
+  MUTEX_FIN((c).waiters_count_lock_); \
+  CloseHandle((c).sema_); \
+} while (0)
+
 # else /* WIN32 */
 /* todo */
 typedef int grn_cond;
@@ -381,6 +410,7 @@ typedef int grn_cond;
   grn_nanosleep(1000000); \
   MUTEX_LOCK(m); \
 } while (0)
+#  define COND_FIN(c)
 /* todo : must be enhanced! */
 
 # endif /* WIN32 */
@@ -392,6 +422,20 @@ typedef int grn_cond;
 
 #endif /* HAVE_PTHREAD_H */
 
+#define MUTEX_LOCK_ENSURE(ctx_, mutex) do {     \
+  grn_ctx *ctx__ = (ctx_);                      \
+  do {                                          \
+    grn_ctx *ctx = ctx__;                       \
+    if (MUTEX_LOCK_CHECK(mutex)) {              \
+      break;                                    \
+    }                                           \
+    if (ctx) {                                  \
+      SERR("MUTEX_LOCK");                       \
+    }                                           \
+    grn_nanosleep(1000000);                     \
+  } while (GRN_TRUE);                           \
+} while (GRN_FALSE)
+
 /* format string for printf */
 #ifdef HAVE_INTTYPES_H
 # include <inttypes.h>
@@ -715,9 +759,9 @@ grn_str_greater(const uint8_t *ap, uint32_t as, const uint8_t *bp, uint32_t bs)
 # endif /* POSIX_HOST_NAME_MAX */
 #endif /* HOST_NAME_MAX */
 
+#define GRN_NEXT_ADDR(p) (((byte *)(p)) + sizeof(*(p)))
+
 GRN_API void grn_sleep(uint32_t seconds);
 GRN_API void grn_nanosleep(uint64_t nanoseconds);
 
 #include <groonga.h>
-
-#endif /* GRN_H */
