@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2009-2015 Brazil
+  Copyright(C) 2009-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -15,8 +15,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GRN_CTX_H
-#define GRN_CTX_H
+
+#pragma once
 
 #include "grn.h"
 #include "grn_error.h"
@@ -45,7 +45,9 @@ extern "C" {
     (ctx)->subno++;\
   } else {\
     (ctx)->errlvl = GRN_OK;\
-    (ctx)->rc = GRN_SUCCESS;\
+    if ((ctx)->rc != GRN_CANCEL) {\
+      (ctx)->rc = GRN_SUCCESS;\
+    }\
     (ctx)->seqno++;\
   }\
   GRN_TEST_YIELD();\
@@ -74,7 +76,9 @@ extern "C" {
 #define ERRCLR(ctx) do {\
   if (ctx) {\
     ((grn_ctx *)ctx)->errlvl = GRN_OK;\
-    ((grn_ctx *)ctx)->rc = GRN_SUCCESS;\
+    if (((grn_ctx *)ctx)->rc != GRN_CANCEL) {\
+      ((grn_ctx *)ctx)->rc = GRN_SUCCESS;\
+    }\
   }\
   errno = 0;\
   grn_gctx.errlvl = GRN_OK;\
@@ -96,10 +100,14 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
   char **p;\
   BACKTRACE(ctx);\
   p = backtrace_symbols((ctx)->trace, (ctx)->ntrace);\
-  for (i = 0; i < (ctx)->ntrace; i++) {\
-    GRN_LOG((ctx), lvl, "%s", p[i]);\
+  if (!p) {\
+    GRN_LOG((ctx), lvl, "backtrace_symbols failed");\
+  } else {\
+    for (i = 0; i < (ctx)->ntrace; i++) {\
+      GRN_LOG((ctx), lvl, "%s", p[i]);\
+    }\
+    free(p);\
   }\
-  free(p);\
 } while (0)
 #else  /* HAVE_BACKTRACE */
 #define LOGTRACE(ctx,msg)
@@ -108,7 +116,9 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
 #define ERRSET(ctx,lvl,r,...) do {\
   grn_ctx *ctx_ = (grn_ctx *)ctx;\
   ctx_->errlvl = (lvl);\
-  ctx_->rc = (r);\
+  if (ctx_->rc != GRN_CANCEL) {\
+    ctx_->rc = (r);\
+  }\
   ctx_->errfile = __FILE__;\
   ctx_->errline = __LINE__;\
   ctx_->errfunc = __FUNCTION__;\
@@ -116,7 +126,6 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
   if (grn_ctx_impl_should_log(ctx)) {\
     grn_ctx_impl_set_current_error_message(ctx);\
     GRN_LOG(ctx, lvl, __VA_ARGS__);\
-    BACKTRACE(ctx);\
     if (lvl <= GRN_LOG_ERROR) { LOGTRACE(ctx, lvl); }\
   }\
 } while (0)
@@ -127,6 +136,7 @@ GRN_API void grn_ctx_impl_set_current_error_message(grn_ctx *ctx);
 #ifdef ERR
 #  undef ERR
 #endif /* ERR */
+#define CRIT(rc,...) ERRSET(ctx, GRN_CRIT, (rc),  __VA_ARGS__)
 #define ERR(rc,...) ERRSET(ctx, GRN_ERROR, (rc),  __VA_ARGS__)
 #define WARN(rc,...) ERRSET(ctx, GRN_WARN, (rc),  __VA_ARGS__)
 #define MERR(...) ERRSET(ctx, GRN_ALERT, GRN_NO_MEMORY_AVAILABLE,  __VA_ARGS__)
@@ -530,6 +540,8 @@ GRN_API void grn_ctx_set_next_expr(grn_ctx *ctx, grn_obj *expr);
 
 int grn_alloc_count(void);
 
+void grn_debug_dump_alloc_info(grn_ctx *ctx);
+
 grn_content_type grn_get_ctype(grn_obj *var);
 
 /**** db_obj ****/
@@ -613,5 +625,3 @@ grn_rc grn_db_init_builtin_procs(grn_ctx *ctx);
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GRN_CTX_H */
