@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2015 Brazil
+/*
+  Copyright(C) 2009-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -14,8 +15,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GRN_DB_H
-#define GRN_DB_H
+
+#pragma once
 
 #include "grn.h"
 #include "grn_ctx.h"
@@ -47,6 +48,7 @@ struct _grn_db {
   grn_db_obj obj;
   grn_obj *keys;
   grn_ja *specs;
+  grn_hash *config;
   grn_tiny_array values;
   grn_critical_section lock;
 };
@@ -81,6 +83,10 @@ grn_rc grn_table_search(grn_ctx *ctx, grn_obj *table,
                         const void *key, uint32_t key_size,
                         grn_operator mode, grn_obj *res, grn_operator op);
 
+grn_rc grn_table_fuzzy_search(grn_ctx *ctx, grn_obj *table,
+                              const void *key, uint32_t key_size,
+                              grn_fuzzy_search_optarg *args, grn_obj *res, grn_operator op);
+
 grn_id grn_table_next(grn_ctx *ctx, grn_obj *table, grn_id id);
 
 int grn_table_get_key2(grn_ctx *ctx, grn_obj *table, grn_id id, grn_obj *bulk);
@@ -137,6 +143,12 @@ struct _grn_type {
   (GRN_OBJ_VECTOR_COLUMNP(obj) &&\
    (DB_OBJ(obj)->header.flags & GRN_OBJ_WITH_WEIGHT))
 
+struct _grn_hook {
+  grn_hook *next;
+  grn_proc *proc;
+  uint32_t hld_size;
+};
+
 typedef struct _grn_proc_ctx grn_proc_ctx;
 
 struct _grn_proc_ctx {
@@ -291,6 +303,8 @@ typedef struct {
   int32_t modify;
 } grn_expr_code;
 
+#define GRN_EXPR_CONST_BLK_SIZE GRN_STACK_SIZE
+
 struct _grn_expr {
   grn_db_obj obj;
   grn_obj name_buf;
@@ -300,7 +314,7 @@ struct _grn_expr {
 
   uint16_t cacheable;
   uint16_t taintable;
-  grn_obj *consts;
+  grn_obj **const_blks;
   grn_obj *values;
   grn_expr_code *codes;
   uint32_t nconsts;
@@ -319,11 +333,13 @@ grn_rc grn_expr_parser_close(grn_ctx *ctx);
 
 /**
  * grn_table_open:
- * @name: 開こうとするtableの名前。NULLなら無名tableとなる。
- * @path: 開こうとするtableのファイルパス。
+ * @name: The table name to be opened. `NULL` means anonymous table.
+ * @path: The path of the table to be opened.
  *
- * ctxが使用するdbの中でnameに対応付けて既存のtableを開く。
- * dbに登録されている名前付きの永続テーブルを開く場合はgrn_ctx_get()を使用するのが望ましい。
+ * Opens an existing table. The table is associated with @name in DB
+ * that is used by @ctx. grn_ctx_get() is better rather than this
+ * function when you want to open a permanent named table that is
+ * registered in DB.
  **/
 GRN_API grn_obj *grn_table_open(grn_ctx *ctx,
                                 const char *name, unsigned int name_size,
@@ -331,13 +347,14 @@ GRN_API grn_obj *grn_table_open(grn_ctx *ctx,
 
 /**
  * grn_column_open:
- * @table: 対象table
- * @name: カラム名
- * @path: カラムを格納するファイルパス。
- * @type: カラム値の型。
+ * @table: The table for the opened column.
+ * @name: The column name to be opened.
+ * @path: The path of the column to be opened.
+ * @type: The type of the column value.
  *
- * 既存の永続的なcolumnを、tableのnameに対応するcolumnとして開く
- * 永続dbに登録されている永続テーブルのカラムを開く場合はgrn_ctx_get()を使用するのが望ましい。
+ * Opens an existing permanent column. The column is associated with
+ * @name in @table. grn_ctx_get() is better rather than this function
+ * when you want to open a column of an permanent table in DB.
  **/
 grn_obj *grn_column_open(grn_ctx *ctx, grn_obj *table,
                          const char *name, unsigned int name_size,
@@ -345,10 +362,10 @@ grn_obj *grn_column_open(grn_ctx *ctx, grn_obj *table,
 
 /**
  * grn_obj_path_rename:
- * @old_path: 旧ファイルパス
- * @new_path: 新ファイルパス
+ * @old_path: The current file path.
+ * @new_path: The new file path.
  *
- * old_pathに該当するオブジェクトのファイル名をnew_pathに変更する。
+ * It renames object's path that is stored in @old_path to @new_path.
  **/
 grn_rc grn_obj_path_rename(grn_ctx *ctx, const char *old_path, const char *new_path);
 
@@ -368,46 +385,6 @@ grn_rc grn_db_obj_init(grn_ctx *ctx, grn_obj *db, grn_id id, grn_db_obj *obj);
 #define GRN_ACCESSORP(obj) \
   ((obj) && (((grn_obj *)(obj))->header.type == GRN_ACCESSOR))
 
-#define GRN_TRUEP(ctx, v, result) do {\
-  switch (v->header.type) {                             \
-  case GRN_BULK :                                       \
-    switch (v->header.domain) {                         \
-    case GRN_DB_BOOL :                                  \
-      result = GRN_BOOL_VALUE(v);                       \
-      break;                                            \
-    case GRN_DB_INT32 :                                 \
-      result = GRN_INT32_VALUE(v) != 0;                 \
-      break;                                            \
-    case GRN_DB_UINT32 :                                \
-      result = GRN_UINT32_VALUE(v) != 0;                \
-      break;                                            \
-    case GRN_DB_FLOAT :                                 \
-      {                                                 \
-        double float_value;                             \
-        float_value = GRN_FLOAT_VALUE(v);               \
-        result = (float_value < -DBL_EPSILON ||         \
-                  DBL_EPSILON < float_value);           \
-      }                                                 \
-      break;                                            \
-    case GRN_DB_SHORT_TEXT :                            \
-    case GRN_DB_TEXT :                                  \
-    case GRN_DB_LONG_TEXT :                             \
-      result = GRN_TEXT_LEN(v) != 0;                    \
-      break;                                            \
-    default :                                           \
-      result = GRN_FALSE;                               \
-      break;                                            \
-    }                                                   \
-    break;                                              \
-  case GRN_VECTOR :                                     \
-    result = GRN_TRUE;                                  \
-    break;                                              \
-  default :                                             \
-    result = GRN_FALSE;                                 \
-    break;                                              \
-  }                                                     \
-} while (0)
-
 grn_id grn_obj_register(grn_ctx *ctx, grn_obj *db, const char *name, unsigned int name_size);
 int grn_obj_is_persistent(grn_ctx *ctx, grn_obj *obj);
 void grn_obj_spec_save(grn_ctx *ctx, grn_db_obj *obj);
@@ -459,8 +436,18 @@ grn_rc grn_accessor_resolve(grn_ctx *ctx, grn_obj *accessor, int deep,
                             grn_obj *base_res, grn_obj **res,
                             grn_search_optarg *optarg);
 
+typedef struct {
+  grn_id target;
+  unsigned int section;
+} grn_obj_default_set_value_hook_data;
+
+grn_obj *grn_obj_default_set_value_hook(grn_ctx *ctx,
+                                        int nargs,
+                                        grn_obj **args,
+                                        grn_user_data *user_data);
+
+void grn_obj_touch(grn_ctx *ctx, grn_obj *obj, grn_timeval *tv);
+
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GRN_DB_H */
