@@ -1,5 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2012 Brazil
+/*
+  Copyright(C) 2009-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -14,8 +15,8 @@
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
-#ifndef GRN_IO_H
-#define GRN_IO_H
+
+#pragma once
 
 #include "grn.h"
 #include "grn_error.h"
@@ -25,17 +26,9 @@ extern "C" {
 #endif
 
 #ifdef WIN32
-#ifdef WIN32_FMO_EACH
-#define GRN_IO_FILE_SIZE  1073741824UL
-#else /* FMO_EACH */
-#define GRN_IO_FILE_SIZE  134217728L
-#endif /* FMO_EACH */
-#define GRN_IO_COPY       grn_io_rdonly
-#define GRN_IO_UPDATE     grn_io_wronly
+# define GRN_IO_FILE_CREATE_MODE (GENERIC_READ | GENERIC_WRITE)
 #else /* WIN32 */
-#define GRN_IO_FILE_SIZE  1073741824UL
-#define GRN_IO_COPY       grn_io_rdwr
-#define GRN_IO_UPDATE     grn_io_rdwr
+# define GRN_IO_FILE_CREATE_MODE 0644
 #endif /* WIN32 */
 
 typedef enum {
@@ -67,9 +60,9 @@ typedef struct {
   void *addr;
   uint32_t diff;
   int32_t cached;
-#if defined(WIN32) && defined(WIN32_FMO_EACH)
+#ifdef WIN32
   HANDLE fmo;
-#endif /* defined(WIN32) && defined(WIN32_FMO_EACH) */
+#endif /* WIN32 */
   void *uncompressed_value;
 } grn_io_win;
 
@@ -77,9 +70,9 @@ typedef struct {
   void *map;
   uint32_t nref;
   uint32_t count;
-#if defined(WIN32) && defined(WIN32_FMO_EACH)
+#ifdef WIN32
   HANDLE fmo;
-#endif /* defined(WIN32) && defined(WIN32_FMO_EACH) */
+#endif /* WIN32 */
 } grn_io_mapinfo;
 
 typedef struct _grn_io_array_info grn_io_array_info;
@@ -130,13 +123,8 @@ GRN_API void *grn_io_header(grn_io *io);
 
 void *grn_io_win_map(grn_io *io, grn_ctx *ctx, grn_io_win *iw, uint32_t segment,
                      uint32_t offset, uint32_t size, grn_io_rw_mode mode);
-grn_rc grn_io_win_mapv(grn_io_win **list, grn_ctx *ctx, int nent);
 grn_rc grn_io_win_unmap(grn_io_win *iw);
 
-void *grn_io_win_map2(grn_io *io, grn_ctx *ctx, grn_io_win *iw, uint32_t segment,
-                      uint32_t offset, uint32_t size, grn_io_rw_mode mode);
-grn_rc grn_io_win_unmap2(grn_io_win *iw);
-
 typedef struct _grn_io_ja_einfo grn_io_ja_einfo;
 typedef struct _grn_io_ja_ehead grn_io_ja_ehead;
 
@@ -363,12 +351,13 @@ uint32_t grn_io_detect_type(grn_ctx *ctx, const char *path);
 grn_rc grn_io_set_type(grn_io *io, uint32_t type);
 uint32_t grn_io_get_type(grn_io *io);
 
-grn_rc grn_io_init(void);
-grn_rc grn_io_fin(void);
+void grn_io_init_from_env(void);
 
 uint32_t grn_io_expire(grn_ctx *ctx, grn_io *io, int count_thresh, uint32_t limit);
 uint32_t grn_expire(grn_ctx *ctx, int count_thresh, uint32_t limit);
 
+grn_rc grn_io_flush(grn_ctx *ctx, grn_io *io);
+
 /* encode/decode */
 
 #define GRN_B_ENC(v,p) do {\
@@ -393,7 +382,7 @@ uint32_t grn_expire(grn_ctx *ctx, int count_thresh, uint32_t limit);
     *_p++ = _v & 0xff; \
   } else { \
     *_p++ = 0x8f; \
-    memcpy(_p, &_v, sizeof(uint32_t));\
+    grn_memcpy(_p, &_v, sizeof(uint32_t));\
     _p += sizeof(uint32_t); \
   } \
   p = _p; \
@@ -408,7 +397,7 @@ uint32_t grn_expire(grn_ctx *ctx, int count_thresh, uint32_t limit);
   switch (_v >> 4) { \
   case 0x08 : \
     if (_v == 0x8f) { \
-      memcpy(&_v, _p, sizeof(uint32_t));\
+      grn_memcpy(&_v, _p, sizeof(uint32_t));\
       _p += sizeof(uint32_t); \
     } \
     break; \
@@ -486,5 +475,3 @@ uint32_t grn_expire(grn_ctx *ctx, int count_thresh, uint32_t limit);
 #ifdef __cplusplus
 }
 #endif
-
-#endif /* GRN_IO_H */
