@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2014 Brazil
+/* Copyright(C) 2009-2015 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,24 +20,79 @@
 #include <string.h>
 #include "grn_str.h"
 #include "grn_db.h"
+#include "grn_expr_code.h"
 #include "grn_util.h"
 #include "grn_output.h"
 
-#define LEVELS (&ctx->impl->levels)
+#define LEVELS (&ctx->impl->output.levels)
 #define DEPTH (GRN_BULK_VSIZE(LEVELS)>>2)
 #define CURR_LEVEL (DEPTH ? (GRN_UINT32_VALUE_AT(LEVELS, (DEPTH - 1))) : 0)
 #define INCR_DEPTH(i) GRN_UINT32_PUT(ctx, LEVELS, i)
 #define DECR_DEPTH (DEPTH ? grn_bulk_truncate(ctx, LEVELS, GRN_BULK_VSIZE(LEVELS) - sizeof(uint32_t)) : 0)
 #define INCR_LENGTH (DEPTH ? (GRN_UINT32_VALUE_AT(LEVELS, (DEPTH - 1)) += 2) : 0)
 
+static void
+indent(grn_ctx *ctx, grn_obj *outbuf, size_t level)
+{
+  size_t i;
+  for (i = 0; i < level; i++) {
+    GRN_TEXT_PUTS(ctx, outbuf, "  ");
+  }
+}
+
+static void
+json_array_open(grn_ctx *ctx, grn_obj *outbuf, size_t *indent_level)
+{
+  GRN_TEXT_PUTC(ctx, outbuf, '[');
+  if (ctx->impl->output.is_pretty) {
+    GRN_TEXT_PUTC(ctx, outbuf, '\n');
+    (*indent_level)++;
+    indent(ctx, outbuf, *indent_level);
+  }
+}
+
+static void
+json_array_close(grn_ctx *ctx, grn_obj *outbuf, size_t *indent_level)
+{
+  if (ctx->impl->output.is_pretty) {
+    GRN_TEXT_PUTC(ctx, outbuf, '\n');
+    (*indent_level)--;
+    indent(ctx, outbuf, *indent_level);
+  }
+  GRN_TEXT_PUTC(ctx, outbuf, ']');
+}
+
+static void
+json_element_end(grn_ctx *ctx, grn_obj *outbuf, size_t indent_level)
+{
+  GRN_TEXT_PUTC(ctx, outbuf, ',');
+  if (ctx->impl->output.is_pretty) {
+    GRN_TEXT_PUTC(ctx, outbuf, '\n');
+    indent(ctx, outbuf, indent_level);
+  }
+}
+
 static void
 put_delimiter(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type)
 {
   uint32_t level = CURR_LEVEL;
   switch (output_type) {
   case GRN_CONTENT_JSON:
-    if (level < 2) { return; }
-    GRN_TEXT_PUTC(ctx, outbuf, ((level & 3) == 3) ? ':' : ',');
+    if (level < 2) {
+      if (DEPTH > 0 && ctx->impl->output.is_pretty) {
+        GRN_TEXT_PUTC(ctx, outbuf, '\n');
+        indent(ctx, outbuf, DEPTH + 1);
+      }
+      return;
+    }
+    if ((level & 3) == 3) {
+      GRN_TEXT_PUTC(ctx, outbuf, ':');
+      if (ctx->impl->output.is_pretty) {
+        GRN_TEXT_PUTC(ctx, outbuf, ' ');
+      }
+    } else {
+      json_element_end(ctx, outbuf, DEPTH + 1);
+    }
     // if (DEPTH == 1 && ((level & 3) != 3)) { GRN_TEXT_PUTC(ctx, outbuf, '\n'); }
     break;
   case GRN_CONTENT_XML:
@@ -74,7 +129,10 @@ grn_output_array_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_typ
     GRN_TEXT_PUTC(ctx, outbuf, '<');
     GRN_TEXT_PUTS(ctx, outbuf, name);
     GRN_TEXT_PUTC(ctx, outbuf, '>');
-    grn_vector_add_element(ctx, &ctx->impl->names, name, strlen(name), 0, GRN_DB_SHORT_TEXT);
+    grn_vector_add_element(ctx,
+                           &ctx->impl->output.names,
+                           name, strlen(name),
+                           0, GRN_DB_SHORT_TEXT);
     break;
   case GRN_CONTENT_TSV:
     if (DEPTH > 2) { GRN_TEXT_PUTS(ctx, outbuf, "[\t"); }
@@ -87,7 +145,7 @@ grn_output_array_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_typ
               nelements,
               name);
     }
-    msgpack_pack_array(&ctx->impl->msgpacker, nelements);
+    msgpack_pack_array(&ctx->impl->output.msgpacker, nelements);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -103,6 +161,10 @@ grn_output_array_close(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_ty
 {
   switch (output_type) {
   case GRN_CONTENT_JSON:
+    if (ctx->impl->output.is_pretty) {
+      GRN_TEXT_PUTC(ctx, outbuf, '\n');
+      indent(ctx, outbuf, DEPTH);
+    }
     GRN_TEXT_PUTC(ctx, outbuf, ']');
     break;
   case GRN_CONTENT_TSV:
@@ -114,7 +176,10 @@ grn_output_array_close(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_ty
   case GRN_CONTENT_XML:
     {
       const char *name;
-      unsigned int name_len = grn_vector_pop_element(ctx, &ctx->impl->names, &name, NULL, NULL);
+      unsigned int name_len;
+      name_len = grn_vector_pop_element(ctx,
+                                        &ctx->impl->output.names,
+                                        &name, NULL, NULL);
       GRN_TEXT_PUTS(ctx, outbuf, "</");
       GRN_TEXT_PUT(ctx, outbuf, name, name_len);
       GRN_TEXT_PUTC(ctx, outbuf, '>');
@@ -145,7 +210,9 @@ grn_output_map_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
     GRN_TEXT_PUTC(ctx, outbuf, '<');
     GRN_TEXT_PUTS(ctx, outbuf, name);
     GRN_TEXT_PUTC(ctx, outbuf, '>');
-    grn_vector_add_element(ctx, &ctx->impl->names, name, strlen(name), 0, GRN_DB_SHORT_TEXT);
+    grn_vector_add_element(ctx,
+                           &ctx->impl->output.names,
+                           name, strlen(name), 0, GRN_DB_SHORT_TEXT);
     break;
   case GRN_CONTENT_TSV:
     if (DEPTH > 2) { GRN_TEXT_PUTS(ctx, outbuf, "{\t"); }
@@ -158,7 +225,7 @@ grn_output_map_open(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
               nelements,
               name);
     }
-    msgpack_pack_map(&ctx->impl->msgpacker, nelements);
+    msgpack_pack_map(&ctx->impl->output.msgpacker, nelements);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -174,6 +241,10 @@ grn_output_map_close(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type
 {
   switch (output_type) {
   case GRN_CONTENT_JSON:
+    if (ctx->impl->output.is_pretty) {
+      GRN_TEXT_PUTC(ctx, outbuf, '\n');
+      indent(ctx, outbuf, DEPTH);
+    }
     GRN_TEXT_PUTS(ctx, outbuf, "}");
     break;
   case GRN_CONTENT_TSV:
@@ -185,7 +256,10 @@ grn_output_map_close(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type
   case GRN_CONTENT_XML:
     {
       const char *name;
-      unsigned int name_len = grn_vector_pop_element(ctx, &ctx->impl->names, &name, NULL, NULL);
+      unsigned int name_len;
+      name_len = grn_vector_pop_element(ctx,
+                                        &ctx->impl->output.names,
+                                        &name, NULL, NULL);
       GRN_TEXT_PUTS(ctx, outbuf, "</");
       GRN_TEXT_PUT(ctx, outbuf, name, name_len);
       GRN_TEXT_PUTC(ctx, outbuf, '>');
@@ -221,7 +295,7 @@ grn_output_int32(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, in
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_int32(&ctx->impl->msgpacker, value);
+    msgpack_pack_int32(&ctx->impl->output.msgpacker, value);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -251,7 +325,7 @@ grn_output_int64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, in
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_int64(&ctx->impl->msgpacker, value);
+    msgpack_pack_int64(&ctx->impl->output.msgpacker, value);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -264,7 +338,7 @@ grn_output_int64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, in
 }
 
 void
-grn_output_uint64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, int64_t value)
+grn_output_uint64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, uint64_t value)
 {
   put_delimiter(ctx, outbuf, output_type);
   switch (output_type) {
@@ -281,7 +355,7 @@ grn_output_uint64(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, i
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_uint64(&ctx->impl->msgpacker, value);
+    msgpack_pack_uint64(&ctx->impl->output.msgpacker, value);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -311,7 +385,7 @@ grn_output_float(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, do
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_double(&ctx->impl->msgpacker, value);
+    msgpack_pack_double(&ctx->impl->output.msgpacker, value);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -342,8 +416,8 @@ grn_output_str(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_raw(&ctx->impl->msgpacker, value_len);
-    msgpack_pack_raw_body(&ctx->impl->msgpacker, value, value_len);
+    msgpack_pack_str(&ctx->impl->output.msgpacker, value_len);
+    msgpack_pack_str_body(&ctx->impl->output.msgpacker, value, value_len);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -381,9 +455,9 @@ grn_output_bool(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, grn
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
     if (value) {
-      msgpack_pack_true(&ctx->impl->msgpacker);
+      msgpack_pack_true(&ctx->impl->output.msgpacker);
     } else {
-      msgpack_pack_false(&ctx->impl->msgpacker);
+      msgpack_pack_false(&ctx->impl->output.msgpacker);
     }
 #endif
     break;
@@ -396,7 +470,7 @@ grn_output_bool(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, grn
   INCR_LENGTH;
 }
 
-static inline void
+void
 grn_output_null(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type)
 {
   put_delimiter(ctx, outbuf, output_type);
@@ -411,7 +485,7 @@ grn_output_null(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type)
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_nil(&ctx->impl->msgpacker);
+    msgpack_pack_nil(&ctx->impl->output.msgpacker);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -453,7 +527,7 @@ grn_output_time(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type, int
     break;
   case GRN_CONTENT_MSGPACK :
 #ifdef GRN_WITH_MESSAGE_PACK
-    msgpack_pack_double(&ctx->impl->msgpacker, dv);
+    msgpack_pack_double(&ctx->impl->output.msgpacker, dv);
 #endif
     break;
   case GRN_CONTENT_GROONGA_COMMAND_LIST :
@@ -510,13 +584,13 @@ grn_output_geo_point(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type
       grn_text_itoa(ctx, &buf, value->latitude);
       GRN_TEXT_PUTC(ctx, &buf, 'x');
       grn_text_itoa(ctx, &buf, value->longitude);
-      msgpack_pack_raw(&ctx->impl->msgpacker, GRN_TEXT_LEN(&buf));
-      msgpack_pack_raw_body(&ctx->impl->msgpacker,
+      msgpack_pack_str(&ctx->impl->output.msgpacker, GRN_TEXT_LEN(&buf));
+      msgpack_pack_str_body(&ctx->impl->output.msgpacker,
                             GRN_TEXT_VALUE(&buf),
                             GRN_TEXT_LEN(&buf));
       grn_obj_close(ctx, &buf);
     } else {
-      msgpack_pack_nil(&ctx->impl->msgpacker);
+      msgpack_pack_nil(&ctx->impl->output.msgpacker);
     }
 #endif
     break;
@@ -565,10 +639,16 @@ grn_text_atoj(grn_ctx *ctx, grn_obj *outbuf, grn_content_type output_type,
       case GRN_ACCESSOR_GET_SCORE :
         {
           grn_rset_recinfo *ri = (grn_rset_recinfo *)grn_obj_get_value_(ctx, a->obj, id, &vs);
-          int32_t int32_score = ri->score;
-          GRN_INT32_PUT(ctx, &buf, int32_score);
+          if (grn_ctx_get_command_version(ctx) == GRN_COMMAND_VERSION_1) {
+            int32_t int32_score = ri->score;
+            GRN_INT32_PUT(ctx, &buf, int32_score);
+            buf.header.domain = GRN_DB_INT32;
+          } else {
+            double float_score = ri->score;
+            GRN_FLOAT_PUT(ctx, &buf, float_score);
+            buf.header.domain = GRN_DB_FLOAT;
+          }
         }
-        buf.header.domain = GRN_DB_INT32;
         break;
       case GRN_ACCESSOR_GET_NSUBRECS :
         {
@@ -1134,57 +1214,6 @@ count_n_elements_in_expression(grn_ctx *ctx, grn_obj *expression)
   return n_elements;
 }
 
-static inline int
-count_used_n_codes(grn_ctx *ctx, grn_expr_code *start, grn_expr_code *target)
-{
-  int n_codes;
-  int i, n_args;
-  grn_bool have_proc_push_code = GRN_FALSE;
-  grn_expr_code *sub_code;
-
-  if (start == target) {
-    return 0;
-  }
-
-  n_args = target->nargs;
-  if (target->op == GRN_OP_CALL) {
-    if (!target->value) {
-      have_proc_push_code = GRN_TRUE;
-    }
-  } else {
-    if (target->value) {
-      n_args--;
-      if (n_args == 0) {
-        return 1;
-      }
-    }
-  }
-
-  n_codes = 1;
-  sub_code = target - 1;
-  for (i = 0; i < n_args; i++) {
-    int sub_n_codes;
-    sub_n_codes = count_used_n_codes(ctx, start, sub_code);
-    n_codes += sub_n_codes;
-    sub_code -= sub_n_codes;
-    if (sub_code < start) {
-      /* TODO: report error */
-      return 0;
-    }
-  }
-
-  if (have_proc_push_code) {
-    n_codes++;
-    sub_code--;
-    if (sub_code < start) {
-      /* TODO: report error */
-      return 0;
-    }
-  }
-
-  return n_codes;
-}
-
 static grn_bool
 is_score_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -1213,7 +1242,11 @@ grn_output_table_column(grn_ctx *ctx, grn_obj *outbuf,
     if (column->header.type == GRN_COLUMN_INDEX) {
       range_id = GRN_DB_UINT32;
     } else if (is_score_accessor(ctx, column)) {
-      range_id = GRN_DB_INT32;
+      if (grn_ctx_get_command_version(ctx) == GRN_COMMAND_VERSION_1) {
+        range_id = GRN_DB_INT32;
+      } else {
+        range_id = GRN_DB_FLOAT;
+      }
     }
     if (range_id == GRN_ID_NIL) {
       range_id = grn_obj_get_range(ctx, column);
@@ -1305,10 +1338,10 @@ grn_output_table_columns_by_expression(grn_ctx *ctx, grn_obj *outbuf,
 
     have_comma = GRN_TRUE;
     if (is_first_comma) {
-      int n_used_codes;
+      unsigned int n_used_codes;
       int code_end_offset;
 
-      n_used_codes = count_used_n_codes(ctx, expr->codes, code - 1);
+      n_used_codes = grn_expr_code_n_used_codes(ctx, expr->codes, code - 1);
       code_end_offset = code - expr->codes - n_used_codes;
 
       grn_output_table_column_by_expression(ctx, outbuf, output_type,
@@ -1417,13 +1450,13 @@ grn_output_table_records_by_expression(grn_ctx *ctx, grn_obj *outbuf,
         have_comma = GRN_TRUE;
         if (is_first_comma) {
           int second_code_offset;
-          int second_code_n_used_code;
+          unsigned int second_code_n_used_codes;
           second_code_offset = code - expr->codes - 1;
-          second_code_n_used_code =
-            count_used_n_codes(ctx,
-                               expr->codes,
-                               expr->codes + second_code_offset);
-          expr->codes_curr = second_code_offset - second_code_n_used_code + 1;
+          second_code_n_used_codes =
+            grn_expr_code_n_used_codes(ctx,
+                                       expr->codes,
+                                       expr->codes + second_code_offset);
+          expr->codes_curr = second_code_offset - second_code_n_used_codes + 1;
           grn_output_table_record_by_expression(ctx, outbuf, output_type,
                                                 format->expression);
           code_start_offset = expr->codes_curr;
@@ -1846,7 +1879,7 @@ typedef struct {
 } msgpack_writer_ctx;
 
 static inline int
-msgpack_buffer_writer(void* data, const char* buf, unsigned int len)
+msgpack_buffer_writer(void* data, const char* buf, msgpack_size_t len)
 {
   msgpack_writer_ctx *writer_ctx = (msgpack_writer_ctx *)data;
   return grn_bulk_write(writer_ctx->ctx, writer_ctx->buffer, buf, len);
@@ -1855,6 +1888,99 @@ msgpack_buffer_writer(void* data, const char* buf, unsigned int len)
 
 #define JSON_CALLBACK_PARAM "callback"
 
+static void
+grn_output_envelope_json(grn_ctx *ctx,
+                         grn_rc rc,
+                         grn_obj *head,
+                         grn_obj *body,
+                         grn_obj *foot,
+                         double started,
+                         double elapsed,
+                         const char *file,
+                         int line)
+{
+  size_t indent_level = 0;
+  grn_obj *expr;
+  grn_obj *jsonp_func = NULL;
+
+  expr = ctx->impl->curr_expr;
+  if (expr) {
+    jsonp_func = grn_expr_get_var(ctx, expr, JSON_CALLBACK_PARAM,
+                                  strlen(JSON_CALLBACK_PARAM));
+  }
+  if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
+    GRN_TEXT_PUT(ctx, head,
+                 GRN_TEXT_VALUE(jsonp_func), GRN_TEXT_LEN(jsonp_func));
+    GRN_TEXT_PUTC(ctx, head, '(');
+  }
+
+  json_array_open(ctx, head, &indent_level);
+  {
+    json_array_open(ctx, head, &indent_level);
+    {
+      grn_text_itoa(ctx, head, rc);
+
+      json_element_end(ctx, head, indent_level);
+      grn_text_ftoa(ctx, head, started);
+
+      json_element_end(ctx, head, indent_level);
+      grn_text_ftoa(ctx, head, elapsed);
+
+      if (rc != GRN_SUCCESS) {
+        json_element_end(ctx, head, indent_level);
+        grn_text_esc(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
+
+        if (ctx->errfunc && ctx->errfile) {
+          grn_obj *command;
+
+          json_element_end(ctx, head, indent_level);
+          json_array_open(ctx, head, &indent_level);
+          {
+            json_array_open(ctx, head, &indent_level);
+            {
+              grn_text_esc(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
+
+              json_element_end(ctx, head, indent_level);
+              grn_text_esc(ctx, head, ctx->errfile, strlen(ctx->errfile));
+
+              json_element_end(ctx, head, indent_level);
+              grn_text_itoa(ctx, head, ctx->errline);
+            }
+            json_array_close(ctx, head, &indent_level);
+
+            if (file && (command = GRN_CTX_USER_DATA(ctx)->ptr)) {
+              json_element_end(ctx, head, indent_level);
+              json_array_open(ctx, head, &indent_level);
+              {
+                grn_text_esc(ctx, head, file, strlen(file));
+
+                json_element_end(ctx, head, indent_level);
+                grn_text_itoa(ctx, head, line);
+
+                json_element_end(ctx, head, indent_level);
+                grn_text_esc(ctx, head,
+                             GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
+              }
+              json_array_close(ctx, head, &indent_level);
+            }
+          }
+          json_array_close(ctx, head, &indent_level);
+        }
+      }
+    }
+    json_array_close(ctx, head, &indent_level);
+  }
+
+  if (GRN_TEXT_LEN(body)) {
+    json_element_end(ctx, head, indent_level);
+  }
+
+  json_array_close(ctx, foot, &indent_level);
+  if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
+    GRN_TEXT_PUTS(ctx, foot, ");");
+  }
+}
+
 void
 grn_output_envelope(grn_ctx *ctx,
                     grn_rc rc,
@@ -1865,8 +1991,6 @@ grn_output_envelope(grn_ctx *ctx,
                     int line)
 {
   double started, finished, elapsed;
-  grn_obj *expr = NULL;
-  grn_obj *jsonp_func = NULL;
 
   grn_timeval tv_now;
   grn_timeval_now(ctx, &tv_now);
@@ -1876,54 +2000,12 @@ grn_output_envelope(grn_ctx *ctx,
   finished += tv_now.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
   elapsed = finished - started;
 
-  switch (ctx->impl->output_type) {
+  switch (ctx->impl->output.type) {
   case GRN_CONTENT_JSON:
-    expr = ctx->impl->curr_expr;
-    if (expr) {
-      jsonp_func = grn_expr_get_var(ctx, expr, JSON_CALLBACK_PARAM,
-                                    strlen(JSON_CALLBACK_PARAM));
-    }
-    if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
-      GRN_TEXT_PUT(ctx, head, GRN_TEXT_VALUE(jsonp_func), GRN_TEXT_LEN(jsonp_func));
-      GRN_TEXT_PUTC(ctx, head, '(');
-    }
-    GRN_TEXT_PUTS(ctx, head, "[[");
-    grn_text_itoa(ctx, head, rc);
-    GRN_TEXT_PUTC(ctx, head, ',');
-    grn_text_ftoa(ctx, head, started);
-    GRN_TEXT_PUTC(ctx, head, ',');
-    grn_text_ftoa(ctx, head, elapsed);
-    if (rc != GRN_SUCCESS) {
-      GRN_TEXT_PUTC(ctx, head, ',');
-      grn_text_esc(ctx, head, ctx->errbuf, strlen(ctx->errbuf));
-      if (ctx->errfunc && ctx->errfile) {
-        grn_obj *command;
-        /* TODO: output backtrace */
-        GRN_TEXT_PUTS(ctx, head, ",[[");
-        grn_text_esc(ctx, head, ctx->errfunc, strlen(ctx->errfunc));
-        GRN_TEXT_PUTC(ctx, head, ',');
-        grn_text_esc(ctx, head, ctx->errfile, strlen(ctx->errfile));
-        GRN_TEXT_PUTC(ctx, head, ',');
-        grn_text_itoa(ctx, head, ctx->errline);
-        GRN_TEXT_PUTS(ctx, head, "]");
-        if (file && (command = GRN_CTX_USER_DATA(ctx)->ptr)) {
-          GRN_TEXT_PUTS(ctx, head, ",[");
-          grn_text_esc(ctx, head, file, strlen(file));
-          GRN_TEXT_PUTC(ctx, head, ',');
-          grn_text_itoa(ctx, head, line);
-          GRN_TEXT_PUTC(ctx, head, ',');
-          grn_text_esc(ctx, head, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
-          GRN_TEXT_PUTS(ctx, head, "]");
-        }
-        GRN_TEXT_PUTS(ctx, head, "]");
-      }
-    }
-    GRN_TEXT_PUTC(ctx, head, ']');
-    if (GRN_TEXT_LEN(body)) { GRN_TEXT_PUTC(ctx, head, ','); }
-    GRN_TEXT_PUTC(ctx, foot, ']');
-    if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
-      GRN_TEXT_PUTS(ctx, foot, ");");
-    }
+    grn_output_envelope_json(ctx, rc,
+                             head, body, foot,
+                             started, elapsed,
+                             file, line);
     break;
   case GRN_CONTENT_TSV:
     grn_text_itoa(ctx, head, rc);
@@ -2024,8 +2106,8 @@ grn_output_envelope(grn_ctx *ctx,
       msgpack_pack_double(&header_packer, elapsed);
 
       if (rc != GRN_SUCCESS) {
-        msgpack_pack_raw(&header_packer, strlen(ctx->errbuf));
-        msgpack_pack_raw_body(&header_packer, ctx->errbuf, strlen(ctx->errbuf));
+        msgpack_pack_str(&header_packer, strlen(ctx->errbuf));
+        msgpack_pack_str_body(&header_packer, ctx->errbuf, strlen(ctx->errbuf));
         if (ctx->errfunc && ctx->errfile) {
           grn_obj *command = GRN_CTX_USER_DATA(ctx)->ptr;
           int error_detail_size;
@@ -2040,27 +2122,27 @@ grn_output_envelope(grn_ctx *ctx,
           }
           msgpack_pack_array(&header_packer, error_detail_size);
 
-          msgpack_pack_raw(&header_packer, strlen(ctx->errfunc));
-          msgpack_pack_raw_body(&header_packer, ctx->errfunc, strlen(ctx->errfunc));
+          msgpack_pack_str(&header_packer, strlen(ctx->errfunc));
+          msgpack_pack_str_body(&header_packer, ctx->errfunc, strlen(ctx->errfunc));
 
-          msgpack_pack_raw(&header_packer, strlen(ctx->errfile));
-          msgpack_pack_raw_body(&header_packer, ctx->errfile, strlen(ctx->errfile));
+          msgpack_pack_str(&header_packer, strlen(ctx->errfile));
+          msgpack_pack_str_body(&header_packer, ctx->errfile, strlen(ctx->errfile));
 
           msgpack_pack_int(&header_packer, ctx->errline);
 
           if (command) {
             if (file) {
-              msgpack_pack_raw(&header_packer, strlen(file));
-              msgpack_pack_raw_body(&header_packer, file, strlen(file));
+              msgpack_pack_str(&header_packer, strlen(file));
+              msgpack_pack_str_body(&header_packer, file, strlen(file));
             } else {
-              msgpack_pack_raw(&header_packer, 7);
-              msgpack_pack_raw_body(&header_packer, "(stdin)", 7);
+              msgpack_pack_str(&header_packer, 7);
+              msgpack_pack_str_body(&header_packer, "(stdin)", 7);
             }
 
             msgpack_pack_int(&header_packer, line);
 
-            msgpack_pack_raw(&header_packer, GRN_TEXT_LEN(command));
-            msgpack_pack_raw_body(&header_packer, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
+            msgpack_pack_str(&header_packer, GRN_TEXT_LEN(command));
+            msgpack_pack_str_body(&header_packer, GRN_TEXT_VALUE(command), GRN_TEXT_LEN(command));
           }
         }
       }
