@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2012-2015 Brazil
+  Copyright(C) 2012-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "grn.h"
+#include "grn_proc.h"
 #include <groonga/plugin.h>
 
 #include <stdarg.h>
@@ -999,6 +1000,13 @@ grn_plugin_malloc(grn_ctx *ctx, size_t size, const char *file, int line,
   return grn_malloc(ctx, size, file, line, func);
 }
 
+void *
+grn_plugin_calloc(grn_ctx *ctx, size_t size, const char *file, int line,
+                  const char *func)
+{
+  return grn_calloc(ctx, size, file, line, func);
+}
+
 void *
 grn_plugin_realloc(grn_ctx *ctx, void *ptr, size_t size,
                    const char *file, int line, const char *func)
@@ -1129,6 +1137,45 @@ grn_plugin_proc_get_var(grn_ctx *ctx, grn_user_data *user_data,
   return grn_proc_get_var(ctx, user_data, name, name_size);
 }
 
+grn_bool
+grn_plugin_proc_get_var_bool(grn_ctx *ctx,
+                             grn_user_data *user_data,
+                             const char *name,
+                             int name_size,
+                             grn_bool default_value)
+{
+  grn_obj *var;
+
+  var = grn_plugin_proc_get_var(ctx, user_data, name, name_size);
+  return grn_proc_option_value_bool(ctx, var, default_value);
+}
+
+int32_t
+grn_plugin_proc_get_var_int32(grn_ctx *ctx,
+                              grn_user_data *user_data,
+                              const char *name,
+                              int name_size,
+                              int32_t default_value)
+{
+  grn_obj *var;
+
+  var = grn_plugin_proc_get_var(ctx, user_data, name, name_size);
+  return grn_proc_option_value_int32(ctx, var, default_value);
+}
+
+const char *
+grn_plugin_proc_get_var_string(grn_ctx *ctx,
+                               grn_user_data *user_data,
+                               const char *name,
+                               int name_size,
+                               size_t *size)
+{
+  grn_obj *var;
+
+  var = grn_plugin_proc_get_var(ctx, user_data, name, name_size);
+  return grn_proc_option_value_string(ctx, var, size);
+}
+
 grn_obj *
 grn_plugin_proc_get_var_by_offset(grn_ctx *ctx, grn_user_data *user_data,
                                   unsigned int offset)
