@@ -255,57 +255,46 @@ func_highlight(grn_ctx *ctx, int nargs, grn_obj **args,
     grn_obj *end_arg = args[nargs - 1];
     int n_args_without_option = nargs;
 
-    if (end_arg->header.type == GRN_PTR) {
-      grn_obj *hash;
-      hash = GRN_PTR_VALUE(end_arg);
+    if (end_arg->header.type == GRN_TABLE_HASH_KEY) {
+      grn_obj *options = end_arg;
+      grn_hash_cursor *cursor;
+      void *key;
+      grn_obj *value;
+      int key_size;
 
-      if (hash) {
-        grn_hash_cursor *cursor;
-        void *key;
-        grn_obj *value;
-        int key_size;
-        if (hash->header.type != GRN_TABLE_HASH_KEY) {
-          GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT,
-                           "highlight(): "
-                           "end argument must be object literal: <%.*s>",
-                           (int)GRN_TEXT_LEN(end_arg),
-                           GRN_TEXT_VALUE(end_arg));
-          goto exit;
-        }
-        n_args_without_option--;
-
-        cursor = grn_hash_cursor_open(ctx, (grn_hash *)hash,
-                                      NULL, 0, NULL, 0,
-                                      0, -1, 0);
-        if (!cursor) {
-          GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
-                           "highlight(): couldn't open cursor");
-          goto exit;
-        }
-        while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
-          grn_hash_cursor_get_key_value(ctx, cursor, &key, &key_size, (void **)&value);
-          if (key_size == 10 && !memcmp(key, "normalizer", 10)) {
-            normalizer_name = GRN_TEXT_VALUE(value);
-            normalizer_name_length = GRN_TEXT_LEN(value);
-          } else if (key_size == 11 && !memcmp(key, "html_escape", 11)) {
-            if (GRN_BOOL_VALUE(value)) {
-              use_html_escape = GRN_TRUE;
-            }
-          } else if (key_size == 16 && !memcmp(key, "default_open_tag", 16)) {
-            default_open_tag = GRN_TEXT_VALUE(value);
-            default_open_tag_length = GRN_TEXT_LEN(value);
-          } else if (key_size == 17 && !memcmp(key, "default_close_tag", 17)) {
-            default_close_tag = GRN_TEXT_VALUE(value);
-            default_close_tag_length = GRN_TEXT_LEN(value);
-          } else {
-            GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT, "invalid option name: <%.*s>",
-                             key_size, (char *)key);
-            grn_hash_cursor_close(ctx, cursor);
-            goto exit;
-          }
-        }
-        grn_hash_cursor_close(ctx, cursor);
+      n_args_without_option--;
+      cursor = grn_hash_cursor_open(ctx, (grn_hash *)options,
+                                    NULL, 0, NULL, 0,
+                                    0, -1, 0);
+      if (!cursor) {
+        GRN_PLUGIN_ERROR(ctx, GRN_NO_MEMORY_AVAILABLE,
+                         "highlight(): couldn't open cursor");
+        goto exit;
       }
+      while (grn_hash_cursor_next(ctx, cursor) != GRN_ID_NIL) {
+        grn_hash_cursor_get_key_value(ctx, cursor, &key, &key_size,
+                                      (void **)&value);
+        if (key_size == 10 && !memcmp(key, "normalizer", 10)) {
+          normalizer_name = GRN_TEXT_VALUE(value);
+          normalizer_name_length = GRN_TEXT_LEN(value);
+        } else if (key_size == 11 && !memcmp(key, "html_escape", 11)) {
+          if (GRN_BOOL_VALUE(value)) {
+            use_html_escape = GRN_TRUE;
+          }
+        } else if (key_size == 16 && !memcmp(key, "default_open_tag", 16)) {
+          default_open_tag = GRN_TEXT_VALUE(value);
+          default_open_tag_length = GRN_TEXT_LEN(value);
+        } else if (key_size == 17 && !memcmp(key, "default_close_tag", 17)) {
+          default_close_tag = GRN_TEXT_VALUE(value);
+          default_close_tag_length = GRN_TEXT_LEN(value);
+        } else {
+          GRN_PLUGIN_ERROR(ctx, GRN_INVALID_ARGUMENT, "invalid option name: <%.*s>",
+                           key_size, (char *)key);
+          grn_hash_cursor_close(ctx, cursor);
+          goto exit;
+        }
+      }
+      grn_hash_cursor_close(ctx, cursor);
     }
 
     keywords =
