@@ -16,10 +16,10 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "grn_proc.h"
+#include "../grn_proc.h"
 
-#include "grn_ctx.h"
-#include "grn_str.h"
+#include "../grn_ctx.h"
+#include "../grn_str.h"
 
 #include <groonga/plugin.h>
 
@@ -30,35 +30,37 @@ command_table_create_parse_flags(grn_ctx *ctx,
 {
   grn_obj_flags flags = 0;
   while (nptr < end) {
+    size_t name_size;
+
     if (*nptr == '|' || *nptr == ' ') {
       nptr += 1;
       continue;
     }
-    if (!memcmp(nptr, "TABLE_HASH_KEY", 14)) {
-      flags |= GRN_OBJ_TABLE_HASH_KEY;
-      nptr += 14;
-    } else if (!memcmp(nptr, "TABLE_PAT_KEY", 13)) {
-      flags |= GRN_OBJ_TABLE_PAT_KEY;
-      nptr += 13;
-    } else if (!memcmp(nptr, "TABLE_DAT_KEY", 13)) {
-      flags |= GRN_OBJ_TABLE_DAT_KEY;
-      nptr += 13;
-    } else if (!memcmp(nptr, "TABLE_NO_KEY", 12)) {
-      flags |= GRN_OBJ_TABLE_NO_KEY;
-      nptr += 12;
-    } else if (!memcmp(nptr, "KEY_NORMALIZE", 13)) {
-      flags |= GRN_OBJ_KEY_NORMALIZE;
-      nptr += 13;
-    } else if (!memcmp(nptr, "KEY_WITH_SIS", 12)) {
-      flags |= GRN_OBJ_KEY_WITH_SIS;
-      nptr += 12;
-    } else {
-      GRN_PLUGIN_ERROR(ctx,
-                       GRN_INVALID_ARGUMENT,
-                       "[table][create][flags] unknown flag: <%.*s>",
-                       (int)(end - nptr), nptr);
-      return 0;
+
+#define CHECK_FLAG(name)                                                \
+    name_size = strlen(#name);                                          \
+    if ((end - nptr) >= name_size &&                                    \
+        memcmp(nptr, #name, name_size) == 0) {                          \
+      flags |= GRN_OBJ_ ## name;                                        \
+      nptr += name_size;                                                \
+      continue;                                                         \
     }
+
+    CHECK_FLAG(TABLE_HASH_KEY);
+    CHECK_FLAG(TABLE_PAT_KEY);
+    CHECK_FLAG(TABLE_DAT_KEY);
+    CHECK_FLAG(TABLE_NO_KEY);
+    CHECK_FLAG(KEY_NORMALIZE);
+    CHECK_FLAG(KEY_WITH_SIS);
+    CHECK_FLAG(KEY_LARGE);
+
+#undef CHECK_FLAG
+
+    GRN_PLUGIN_ERROR(ctx,
+                     GRN_INVALID_ARGUMENT,
+                     "[table][create][flags] unknown flag: <%.*s>",
+                     (int)(end - nptr), nptr);
+    return 0;
   }
   return flags;
 }
@@ -474,19 +476,42 @@ command_table_remove(grn_ctx *ctx,
 {
   grn_obj *name;
   grn_obj *table;
+  grn_bool dependent;
 
   name = grn_plugin_proc_get_var(ctx, user_data, "name", -1);
+  dependent = grn_plugin_proc_get_var_bool(ctx, user_data, "dependent", -1,
+                                           GRN_FALSE);
   table = grn_ctx_get(ctx,
                       GRN_TEXT_VALUE(name),
                       GRN_TEXT_LEN(name));
-  if (table) {
-    grn_obj_remove(ctx,table);
-  } else {
+  if (!table) {
     GRN_PLUGIN_ERROR(ctx,
                      GRN_INVALID_ARGUMENT,
                      "[table][remove] table isn't found: <%.*s>",
                      (int)GRN_TEXT_LEN(name),
                      GRN_TEXT_VALUE(name));
+    grn_ctx_output_bool(ctx, GRN_FALSE);
+    return NULL;
+  }
+
+  if (!grn_obj_is_table(ctx, table)) {
+    const char *type_name;
+    type_name = grn_obj_type_to_string(table->header.type);
+    grn_obj_unlink(ctx, table);
+    GRN_PLUGIN_ERROR(ctx,
+                     GRN_INVALID_ARGUMENT,
+                     "[table][remove] not table: <%.*s>: <%s>",
+                     (int)GRN_TEXT_LEN(name),
+                     GRN_TEXT_VALUE(name),
+                     type_name);
+    grn_ctx_output_bool(ctx, GRN_FALSE);
+    return NULL;
+  }
+
+  if (dependent) {
+    grn_obj_remove_dependent(ctx, table);
+  } else {
+    grn_obj_remove(ctx, table);
   }
   grn_ctx_output_bool(ctx, !ctx->rc);
   return NULL;
@@ -495,13 +520,14 @@ command_table_remove(grn_ctx *ctx,
 void
 grn_proc_init_table_remove(grn_ctx *ctx)
 {
-  grn_expr_var vars[1];
+  grn_expr_var vars[2];
 
   grn_plugin_expr_var_init(ctx, &(vars[0]), "name", -1);
+  grn_plugin_expr_var_init(ctx, &(vars[1]), "dependent", -1);
   grn_plugin_command_create(ctx,
                             "table_remove", -1,
                             command_table_remove,
-                            1,
+                            2,
                             vars);
 }
 
