@@ -1,5 +1,5 @@
 /* -*- c-basic-offset: 2 -*- */
-/* Copyright(C) 2009-2015 Brazil
+/* Copyright(C) 2009-2016 Brazil
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -16,7 +16,6 @@
 */
 #include "grn.h"
 #include <limits.h>
-#include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
 #include "grn_db.h"
@@ -28,7 +27,11 @@
 #include <math.h>
 
 #if defined(HAVE__GMTIME64_S) && defined(__GNUC__)
-# define gmtime_s(tm, time) _gmtime64_s(tm, time)
+# ifdef _WIN64
+#  define gmtime_s(tm, time) _gmtime64_s(tm, time)
+# else /* _WIN64 */
+#  define gmtime_s(tm, time) _gmtime32_s(tm, time)
+# endif /* _WIN64 */
 #endif /* defined(HAVE__GMTIME64_S) && defined(__GNUC__) */
 
 /* For Visual C++ 2010. Drop the code when we drop Visual C++ 2010 support. */
@@ -91,7 +94,6 @@ grn_str_charlen(grn_ctx *ctx, const char *str, grn_encoding encoding)
       }
     }
     return 1;
-    break;
   case GRN_ENC_UTF8 :
     if (*p & 0x80) {
       int b, w;
@@ -111,7 +113,6 @@ grn_str_charlen(grn_ctx *ctx, const char *str, grn_encoding encoding)
     } else {
       return 1;
     }
-    break;
   case GRN_ENC_SJIS :
     if (*p & 0x80) {
       /* we regard 0xa0 as JIS X 0201 KANA. adjusted to other tools. */
@@ -128,10 +129,8 @@ grn_str_charlen(grn_ctx *ctx, const char *str, grn_encoding encoding)
     } else {
       return 1;
     }
-    break;
   default :
     return 1;
-    break;
   }
   return 0;
 }
@@ -155,10 +154,8 @@ grn_charlen_(grn_ctx *ctx, const char *str, const char *end, grn_encoding encodi
       }
     }
     return 1;
-    break;
   case GRN_ENC_UTF8 :
     return grn_str_charlen_utf8(ctx, p, (unsigned char *)end);
-    break;
   case GRN_ENC_SJIS :
     if (*p & 0x80) {
       /* we regard 0xa0 as JIS X 0201 KANA. adjusted to other tools. */
@@ -175,10 +172,8 @@ grn_charlen_(grn_ctx *ctx, const char *str, const char *end, grn_encoding encodi
     } else {
       return 1;
     }
-    break;
   default :
     return 1;
-    break;
   }
   return 0;
 }
@@ -534,7 +529,7 @@ normalize_utf8(grn_ctx *ctx, grn_str *nstr)
             nstr->ctypes = ctypes;
           }
         }
-        memcpy(d, p, lp);
+        grn_memcpy(d, p, lp);
         d_ = d;
         d += lp;
         length++;
@@ -1150,7 +1145,7 @@ grn_fakenstr_open(grn_ctx *ctx, const char *str, size_t str_len, grn_encoding en
   }
   nstr->orig = str;
   nstr->orig_blen = str_len;
-  memcpy(nstr->norm, str, str_len);
+  grn_memcpy(nstr->norm, str, str_len);
   nstr->norm[str_len] = '\0';
   nstr->norm_blen = str_len;
   nstr->ctypes = NULL;
@@ -1496,6 +1491,20 @@ grn_atoll(const char *nptr, const char *end, const char **rest)
   return n ? -v : v;
 }
 
+uint64_t
+grn_atoull(const char *nptr, const char *end, const char **rest)
+{
+  uint64_t v = 0, t;
+  while (nptr < end && *nptr >= '0' && *nptr <= '9') {
+    t = v * 10 + (*nptr - '0');
+    if (t < v) { v = 0; break; }
+    v = t;
+    nptr++;
+  }
+  if (rest) { *rest = nptr; }
+  return v;
+}
+
 unsigned int
 grn_htoui(const char *nptr, const char *end, const char **rest)
 {
@@ -1776,7 +1785,6 @@ grn_aton(grn_ctx *ctx, const char *p, const char *end, const char **rest,
     break;
   default :
     return GRN_INVALID_ARGUMENT;
-    break;
   }
 
   return GRN_SUCCESS;
@@ -1920,7 +1928,7 @@ grn_bulk_resize(grn_ctx *ctx, grn_obj *buf, unsigned int newsize)
       if (rounded_newsize < newsize) { return GRN_NOT_ENOUGH_SPACE; }
       newsize = rounded_newsize;
       if (!(head = GRN_MALLOC(newsize))) { return GRN_NO_MEMORY_AVAILABLE; }
-      memcpy(head, GRN_BULK_HEAD(buf), GRN_BULK_VSIZE(buf));
+      grn_memcpy(head, GRN_BULK_HEAD(buf), GRN_BULK_VSIZE(buf));
       buf->u.b.curr = head + grn_bulk_margin_size + GRN_BULK_VSIZE(buf);
       buf->u.b.head = head + grn_bulk_margin_size;
       buf->u.b.tail = head + newsize;
@@ -1946,7 +1954,7 @@ grn_bulk_write(grn_ctx *ctx, grn_obj *buf, const char *str, unsigned int len)
     if ((rc = grn_bulk_resize(ctx, buf, GRN_BULK_VSIZE(buf) + len))) { return rc; }
   }
   curr = GRN_BULK_CURR(buf);
-  memcpy(curr, str, len);
+  grn_memcpy(curr, str, len);
   GRN_BULK_INCR_LEN(buf, len);
   return rc;
 }
@@ -2078,26 +2086,30 @@ grn_text_ulltoa(grn_ctx *ctx, grn_obj *buf, unsigned long long int i)
 inline static void
 ftoa_(grn_ctx *ctx, grn_obj *buf, double d)
 {
-  char *curr;
+  char *start;
+  size_t before_size;
   size_t len;
 #define DIGIT_NUMBER 15
-  grn_bulk_reserve(ctx, buf, DIGIT_NUMBER + 1);
-  curr = GRN_BULK_CURR(buf);
-  len = sprintf(curr, "%#.*g", DIGIT_NUMBER, d);
+#define FIRST_BUFFER_SIZE (DIGIT_NUMBER + 4)
+  before_size = GRN_BULK_VSIZE(buf);
+  grn_bulk_reserve(ctx, buf, FIRST_BUFFER_SIZE);
+  grn_text_printf(ctx, buf, "%#.*g", DIGIT_NUMBER, d);
+  len = GRN_BULK_VSIZE(buf) - before_size;
+  start = GRN_BULK_CURR(buf) - len;
+#undef FIRST_BUFFER_SIZE
 #undef DIGIT_NUMBER
-  if (curr[len - 1] == '.') {
-    GRN_BULK_INCR_LEN(buf, len);
+  if (start[len - 1] == '.') {
     GRN_TEXT_PUTC(ctx, buf, '0');
   } else {
     char *p, *q;
-    curr[len] = '\0';
-    if ((p = strchr(curr, 'e'))) {
+    start[len] = '\0';
+    if ((p = strchr(start, 'e'))) {
       for (q = p; *(q - 2) != '.' && *(q - 1) == '0'; q--) { len--; }
-      memmove(q, p, curr + len - q);
+      grn_memmove(q, p, start + len - q);
     } else {
-      for (q = curr + len; *(q - 2) != '.' && *(q - 1) == '0'; q--) { len--; }
+      for (q = start + len; *(q - 2) != '.' && *(q - 1) == '0'; q--) { len--; }
     }
-    GRN_BULK_INCR_LEN(buf, len);
+    grn_bulk_truncate(ctx, buf, before_size + len);
   }
 }
 
@@ -2495,9 +2507,11 @@ grn_text_printf(grn_ctx *ctx, grn_obj *bulk, const char *format, ...)
 grn_rc
 grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk, const char *format, va_list args)
 {
-  int rest_size, written_size;
+  grn_bool is_written = GRN_FALSE;
+  int written_size;
 
   {
+    int rest_size;
     va_list copied_args;
 
     rest_size = GRN_BULK_REST(bulk);
@@ -2505,9 +2519,43 @@ grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk, const char *format, va_list args)
     written_size = vsnprintf(GRN_BULK_CURR(bulk), rest_size,
                              format, copied_args);
     va_end(copied_args);
+
+    if (written_size < rest_size) {
+      is_written = GRN_TRUE;
+    }
   }
 
-  if (written_size >= rest_size) {
+#ifdef WIN32
+  if (written_size == -1 && errno == ERANGE) {
+# define N_NEW_SIZES 3
+    int i;
+    int new_sizes[N_NEW_SIZES];
+
+    new_sizes[0] = GRN_BULK_REST(bulk) + strlen(format) * 2;
+    new_sizes[1] = new_sizes[0] + 4096;
+    new_sizes[2] = new_sizes[0] + 65536;
+
+    for (i = 0; i < N_NEW_SIZES; i++) {
+      grn_rc rc;
+      int new_size = new_sizes[i];
+      va_list copied_args;
+
+      rc = grn_bulk_reserve(ctx, bulk, GRN_BULK_VSIZE(bulk) + new_size);
+      if (rc) {
+        return rc;
+      }
+      va_copy(copied_args, args);
+      written_size = vsnprintf(GRN_BULK_CURR(bulk), new_size,
+                               format, copied_args);
+      va_end(copied_args);
+      if (written_size != -1) {
+        break;
+      }
+    }
+# undef N_NEW_SIZES
+  }
+#else /* WIN32 */
+  if (!is_written) {
     grn_rc rc;
     int required_size = written_size + 1; /* "+ 1" for terminate '\0'. */
 
@@ -2518,6 +2566,7 @@ grn_text_vprintf(grn_ctx *ctx, grn_obj *bulk, const char *format, va_list args)
     written_size = vsnprintf(GRN_BULK_CURR(bulk), required_size,
                              format, args);
   }
+#endif /* WIN32 */
 
   if (written_size < 0) {
     return GRN_INVALID_ARGUMENT;
@@ -3152,7 +3201,7 @@ grn_str_url_path_normalize(grn_ctx *ctx, const char *path, size_t path_len,
       }
     }
     if (be - b >= pc - p) {
-      memcpy(b, p, (pc - p));
+      grn_memcpy(b, p, (pc - p));
       b += pc - p;
       p = pc;
       if (p < pe && *pc == '/' && be > b) {
@@ -3164,22 +3213,6 @@ grn_str_url_path_normalize(grn_ctx *ctx, const char *path, size_t path_len,
   *b = '\0';
 }
 
-grn_rc
-grn_text_fgets(grn_ctx *ctx, grn_obj *buf, FILE *fp)
-{
-  size_t len;
-  grn_rc rc = GRN_END_OF_DATA;
-  for (;;) {
-    grn_bulk_reserve(ctx, buf, BUFSIZ);
-    if (!fgets(GRN_BULK_CURR(buf), BUFSIZ, fp)) { break; }
-    if (!(len = strlen(GRN_BULK_CURR(buf)))) { break; }
-    GRN_BULK_INCR_LEN(buf, len);
-    rc = GRN_SUCCESS;
-    if (GRN_BULK_CURR(buf)[-1] == '\n') { break; }
-  }
-  return rc;
-}
-
 grn_bool
 grn_bulk_is_zero(grn_ctx *ctx, grn_obj *obj)
 {
