@@ -16,16 +16,16 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "lib/com.h"
-#include "lib/ctx_impl.h"
+#include <grn_com.h>
+#include <grn_ctx_impl.h>
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
+# include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif /* HAVE_NETINET_IN_H */
+#ifndef WIN32
+# include <netinet/in.h>
+#endif /* WIN32 */
 
 #define DEFAULT_PORT 10041
 #define DEFAULT_HOST "localhost"
@@ -54,7 +54,7 @@ lprint(grn_ctx *ctx, const char *fmt, ...)
   int len;
   va_list argp;
   grn_timeval_now(ctx, &tv);
-  grn_timeval2str(ctx, &tv, buf);
+  grn_timeval2str(ctx, &tv, buf, 1024);
   len = strlen(buf);
   buf[len++] = '|';
   va_start(argp, fmt);
@@ -196,7 +196,7 @@ msg_handler(grn_ctx *ctx, grn_obj *msg)
   grn_msg_close(ctx, msg);
 }
 
-static void * CALLBACK
+static grn_thread_func_result CALLBACK
 receiver(void *arg)
 {
   grn_ctx ctx_, *ctx = &ctx_;
@@ -213,7 +213,7 @@ receiver(void *arg)
     */
   }
   grn_ctx_fin(ctx);
-  return NULL;
+  return GRN_THREAD_FUNC_RETURN_VALUE;
 }
 
 static int
@@ -280,7 +280,7 @@ do_client()
             if ((nrecv < max_tp * (tve.tv_sec - tvb.tv_sec)) &&
                 (nsent - nrecv) < max_con) { break; }
             /* lprint(ctx, "s:%d r:%d", nsent, nrecv); */
-            usleep(1000);
+            grn_nanosleep(1000000);
           }
           if (!(nsent % 1000)) { lprint(ctx, "     : %d", nsent); }
         }
@@ -329,11 +329,11 @@ main(int argc, char **argv)
   const char *protostr = NULL, *maxconstr = NULL, *maxtpstr = NULL;
   int r, i, flags = 0;
   static grn_str_getopt_opt opts[] = {
-    {'P', NULL, NULL, 0, getopt_op_none},
-    {'m', NULL, NULL, 0, getopt_op_none},
-    {'t', NULL, NULL, 0, getopt_op_none},
-    {'h', NULL, NULL, flag_usage, getopt_op_on},
-    {'v', NULL, NULL, flag_verbose, getopt_op_on},
+    {'P', NULL, NULL, 0, GETOPT_OP_NONE},
+    {'m', NULL, NULL, 0, GETOPT_OP_NONE},
+    {'t', NULL, NULL, 0, GETOPT_OP_NONE},
+    {'h', NULL, NULL, flag_usage, GETOPT_OP_ON},
+    {'v', NULL, NULL, flag_verbose, GETOPT_OP_ON},
     {'\0', NULL, NULL, 0, 0}
   };
   opts[0].arg = &protostr;
@@ -362,6 +362,8 @@ main(int argc, char **argv)
     flags |= flag_usage;
   }
 
+  grn_default_logger_set_path(GRN_LOG_PATH);
+
   if (grn_init()) { return -1; }
   if (flags & flag_usage) {
     usage(); r = -1;
