@@ -16,7 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <dat.h>
+#include <grn_dat.h>
 
 #include <gcutter.h>
 #include <glib/gstdio.h>
