@@ -13,10 +13,10 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "str.h"
+#include "grn_str.h"
 #include <stdio.h>
 
 #include <gcutter.h>
@@ -84,8 +84,8 @@ test_no_columns(void)
   assert_send_command("table_create Users TABLE_HASH_KEY ShortText");
   assert_send_command("load --table Users\n"
                       "[\n"
-                      "{\"_key\":\"mori\"}\n"
-                      "{\"_key\":\"gunyara-kun\"}\n"
+                      "{\"_key\":\"mori\"},\n"
+                      "{\"_key\":\"gunyara-kun\"},\n"
                       "{\"_key\":\"yu\"}\n"
                       "]");
   cut_assert_equal_string(
