@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2; coding: utf-8 -*- */
 /*
-  Copyright (C) 2011-2015  Kouhei Sutou <kou@clear-code.com>
+  Copyright (C) 2011-2016  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -29,12 +29,18 @@ void data_is_builtin(void);
 void test_is_builtin(gconstpointer data);
 void data_is_table(void);
 void test_is_table(gconstpointer data);
+void data_is_column(void);
+void test_is_column(gconstpointer data);
+void data_is_reference_column(void);
+void test_is_reference_column(gconstpointer data);
 void data_is_accessor(void);
 void test_is_accessor(gconstpointer data);
 void data_is_key_accessor(void);
 void test_is_key_accessor(gconstpointer data);
 void data_is_type(void);
 void test_is_type(gconstpointer data);
+void data_is_text_family_type(void);
+void test_is_text_family_type(gconstpointer data);
 void data_is_proc(void);
 void test_is_proc(gconstpointer data);
 void data_is_tokenizer_proc(void);
@@ -49,6 +55,8 @@ void data_is_token_filter_proc(void);
 void test_is_token_filter_proc(gconstpointer data);
 void data_is_scorer_proc(void);
 void test_is_scorer_proc(gconstpointer data);
+void data_type_to_string(void);
+void test_type_to_string(gconstpointer data);
 
 static gchar *tmp_directory;
 static const gchar *database_path;
@@ -173,6 +181,80 @@ test_is_table(gconstpointer data)
   }
 }
 
+void
+data_is_column(void)
+{
+#define ADD_DATUM(label, expected, name)                                \
+  gcut_add_datum(label,                                                 \
+                 "expected", G_TYPE_BOOLEAN, expected,                  \
+                 "name", G_TYPE_STRING, name,                           \
+                 NULL)
+
+  ADD_DATUM("table",           FALSE, "Users");
+  ADD_DATUM("fix size column", TRUE,  "Users.age");
+  ADD_DATUM("var size column", TRUE,  "Users.name");
+  ADD_DATUM("index column",    TRUE,  "Names.users");
+
+#undef ADD_DATUM
+}
+
+void
+test_is_column(gconstpointer data)
+{
+  const gchar *name;
+  grn_obj *object;
+
+  assert_send_command("table_create Users TABLE_HASH_KEY ShortText");
+  assert_send_command("column_create Users age COLUMN_SCALAR UInt8");
+  assert_send_command("column_create Users name COLUMN_SCALAR ShortText");
+  assert_send_command("table_create Names TABLE_PAT_KEY ShortText");
+  assert_send_command("column_create Names users COLUMN_INDEX Users name");
+
+  name = gcut_data_get_string(data, "name");
+  object = grn_ctx_get(context, name, strlen(name));
+  if (gcut_data_get_string(data, "expected")) {
+    cut_assert_true(grn_obj_is_column(context, object));
+  } else {
+    cut_assert_false(grn_obj_is_column(context, object));
+  }
+}
+
+void
+data_is_reference_column(void)
+{
+#define ADD_DATUM(label, expected, name)                                \
+  gcut_add_datum(label,                                                 \
+                 "expected", G_TYPE_BOOLEAN, expected,                  \
+                 "name", G_TYPE_STRING, name,                           \
+                 NULL)
+
+  ADD_DATUM("table",            FALSE, "Users");
+  ADD_DATUM("value column",     FALSE, "Users.age");
+  ADD_DATUM("reference column", TRUE,  "Users.name");
+
+#undef ADD_DATUM
+}
+
+void
+test_is_reference_column(gconstpointer data)
+{
+  const gchar *name;
+  grn_obj *object;
+
+  assert_send_command("table_create Names TABLE_PAT_KEY ShortText");
+  assert_send_command("table_create Users TABLE_HASH_KEY ShortText");
+  assert_send_command("column_create Users age COLUMN_SCALAR UInt8");
+  assert_send_command("column_create Users name COLUMN_SCALAR Names");
+
+  name = gcut_data_get_string(data, "name");
+  object = grn_ctx_get(context, name, strlen(name));
+  if (gcut_data_get_string(data, "expected")) {
+    cut_assert_true(grn_obj_is_reference_column(context, object));
+  } else {
+    cut_assert_false(grn_obj_is_reference_column(context, object));
+  }
+}
+
 void
 data_is_accessor(void)
 {
@@ -282,6 +364,41 @@ test_is_type(gconstpointer data)
   }
 }
 
+void
+data_is_text_family_type(void)
+{
+#define ADD_DATUM(expected, name)                                       \
+  gcut_add_datum((expected ?                                            \
+                  "type - " name :                                      \
+                  "not type - " name),                                  \
+                 "expected", G_TYPE_BOOLEAN, expected,                  \
+                 "name", G_TYPE_STRING, name,                           \
+                 NULL)
+
+  ADD_DATUM(FALSE, "Time");
+  ADD_DATUM(TRUE, "ShortText");
+  ADD_DATUM(TRUE, "Text");
+  ADD_DATUM(TRUE, "LongText");
+  ADD_DATUM(FALSE, "TokyoGeoPoint");
+
+#undef ADD_DATUM
+}
+
+void
+test_is_text_family_type(gconstpointer data)
+{
+  const gchar *name;
+  grn_obj *object;
+
+  name = gcut_data_get_string(data, "name");
+  object = grn_ctx_get(context, name, strlen(name));
+  if (gcut_data_get_string(data, "expected")) {
+    cut_assert_true(grn_obj_is_text_family_type(context, object));
+  } else {
+    cut_assert_false(grn_obj_is_text_family_type(context, object));
+  }
+}
+
 void
 data_is_proc(void)
 {
@@ -509,3 +626,58 @@ test_is_scorer_proc(gconstpointer data)
     cut_assert_false(grn_obj_is_scorer_proc(context, object));
   }
 }
+
+void
+data_type_to_string(void)
+{
+#define ADD_DATUM(expected, type)                                      \
+  gcut_add_datum(G_STRINGIFY(type),                                    \
+                 "expected", G_TYPE_STRING, expected,                  \
+                 "type", G_TYPE_UINT, type,                            \
+                 NULL)
+
+  ADD_DATUM("void", GRN_VOID);
+  ADD_DATUM("bulk", GRN_BULK);
+  ADD_DATUM("ptr", GRN_PTR);
+  ADD_DATUM("uvector", GRN_UVECTOR);
+  ADD_DATUM("pvector", GRN_PVECTOR);
+  ADD_DATUM("vector", GRN_VECTOR);
+  ADD_DATUM("msg", GRN_MSG);
+  ADD_DATUM("query", GRN_QUERY);
+  ADD_DATUM("accessor", GRN_ACCESSOR);
+  ADD_DATUM("snip", GRN_SNIP);
+  ADD_DATUM("patsnip", GRN_PATSNIP);
+  ADD_DATUM("string", GRN_STRING);
+  ADD_DATUM("cursor:table:hash_key", GRN_CURSOR_TABLE_HASH_KEY);
+  ADD_DATUM("cursor:table:pat_key", GRN_CURSOR_TABLE_PAT_KEY);
+  ADD_DATUM("cursor:table:dat_key", GRN_CURSOR_TABLE_DAT_KEY);
+  ADD_DATUM("cursor:table:no_key", GRN_CURSOR_TABLE_NO_KEY);
+  ADD_DATUM("cursor:column:index", GRN_CURSOR_COLUMN_INDEX);
+  ADD_DATUM("cursor:column:geo_index", GRN_CURSOR_COLUMN_GEO_INDEX);
+  ADD_DATUM("cursor:config", GRN_CURSOR_CONFIG);
+  ADD_DATUM("type", GRN_TYPE);
+  ADD_DATUM("proc", GRN_PROC);
+  ADD_DATUM("expr", GRN_EXPR);
+  ADD_DATUM("table:hash_key", GRN_TABLE_HASH_KEY);
+  ADD_DATUM("table:pat_key", GRN_TABLE_PAT_KEY);
+  ADD_DATUM("table:dat_key", GRN_TABLE_DAT_KEY);
+  ADD_DATUM("table:no_key", GRN_TABLE_NO_KEY);
+  ADD_DATUM("db", GRN_DB);
+  ADD_DATUM("column:fix_size", GRN_COLUMN_FIX_SIZE);
+  ADD_DATUM("column:var_size", GRN_COLUMN_VAR_SIZE);
+  ADD_DATUM("column:index", GRN_COLUMN_INDEX);
+
+#undef ADD_DATUM
+}
+
+void
+test_type_to_string(gconstpointer data)
+{
+  const gchar *expected;
+  guint type;
+
+  expected = gcut_data_get_string(data, "expected");
+  type = gcut_data_get_uint(data, "type");
+  cut_assert_equal_string(expected,
+                          grn_obj_type_to_string(type));
+}
