@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <groonga.h>
@@ -382,6 +382,7 @@ data_create_with_valid_name(void)
 #define ADD_DATUM_VALID_CHARACTER(name)               \
   ADD_DATUM(name, name)
 
+  ADD_DATUM_VALID_CHARACTER("@");
   ADD_DATUM_VALID_CHARACTER("#");
   ADD_DATUM_VALID_CHARACTER("-");
 
@@ -438,7 +439,6 @@ data_create_with_invalid_name(void)
   ADD_DATUM_INVALID_CHARACTER("=");
   ADD_DATUM_INVALID_CHARACTER(">");
   ADD_DATUM_INVALID_CHARACTER("?");
-  ADD_DATUM_INVALID_CHARACTER("@");
   ADD_DATUM_INVALID_CHARACTER("[");
   ADD_DATUM_INVALID_CHARACTER("\\");
   ADD_DATUM_INVALID_CHARACTER("]");
@@ -468,8 +468,8 @@ test_create_with_invalid_name(gpointer data)
                            NULL, NULL);
   grn_test_assert_error(
     GRN_INVALID_ARGUMENT,
-    cut_take_printf("[table][create]: name can't start with '_' and "
-                    "contains only 0-9, A-Z, a-z, #, - or _: <%s>",
+    cut_take_printf("[table][create] name can't start with '_' and "
+                    "contains only 0-9, A-Z, a-z, #, @, - or _: <%s>",
                     table_name),
     context);
 }
