@@ -16,8 +16,8 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include <str.h>
-#include <db.h>
+#include <grn_str.h>
+#include <grn_db.h>
 
 #include "grn-assertions.h"
 
